@@ -1,7 +1,7 @@
 // qtractor.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -28,6 +28,8 @@
 #include <QTranslator>
 #include <QLocale>
 
+#include <QDir>
+
 #include <QStyleFactory>
 
 
@@ -40,6 +42,20 @@
 #define CONFIG_DATADIR CONFIG_PREFIX "/share"
 #endif
 
+#ifndef CONFIG_LIBDIR
+#if defined(__x86_64__)
+#define CONFIG_LIBDIR CONFIG_PREFIX "/lib64"
+#else
+#define CONFIG_LIBDIR CONFIG_PREFIX "/lib"
+#endif
+#endif
+
+#if QT_VERSION < 0x050000
+#define CONFIG_PLUGINSDIR CONFIG_LIBDIR "/qt4/plugins"
+#else
+#define CONFIG_PLUGINSDIR CONFIG_LIBDIR "/qt5/plugins"
+#endif
+
 
 //-------------------------------------------------------------------------
 // Singleton application instance stuff (Qt/X11 only atm.)
@@ -483,6 +499,10 @@ int main ( int argc, char **argv )
 		return 2;
 	}
 
+	// Special style paths...
+	if (QDir(CONFIG_PLUGINSDIR).exists())
+		app.addLibraryPath(CONFIG_PLUGINSDIR);
+
 	// Custom style theme...
 	if (!options.sCustomStyleTheme.isEmpty())
 		app.setStyle(QStyleFactory::create(options.sCustomStyleTheme));
