@@ -1,7 +1,7 @@
 // qtractorAudioEngine.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -461,6 +461,9 @@ qtractorAudioEngine::qtractorAudioEngine ( qtractorSession *pSession )
 
 	// JACK transport mode.
 	m_transportMode = qtractorBus::Duplex;
+
+	// JACK timebase mode.
+	m_bTimebase = true;
 }
 
 
@@ -629,8 +632,8 @@ bool qtractorAudioEngine::activate (void)
 	jack_set_process_callback(m_pJackClient,
 			qtractorAudioEngine_process, this);
 
-	// Trnsport timebase callbacks...
-	if (m_transportMode & qtractorBus::Output) {
+	// Transport timebase callbacks...
+	if (m_bTimebase) {
 		jack_set_timebase_callback(m_pJackClient,
 			0 /* FIXME: un-conditional! */,
 			qtractorAudioEngine_timebase, this);
@@ -645,7 +648,7 @@ bool qtractorAudioEngine::activate (void)
 		qtractorAudioEngine_graph_order, this);
 	jack_set_port_registration_callback(m_pJackClient,
 		qtractorAudioEngine_graph_port, this);
-    jack_set_buffer_size_callback(m_pJackClient,
+	jack_set_buffer_size_callback(m_pJackClient,
 		qtractorAudioEngine_buffer_size, this);
 
 	// Set audio export processor callback.
@@ -672,8 +675,8 @@ bool qtractorAudioEngine::activate (void)
 		qtractorAudioEngine_property_change, this);
 #endif
 
-    // Reset all dependable monitoring...
-    resetAllMonitors();
+	// Reset all dependable monitoring...
+	resetAllMonitors();
 
 	// Time to activate ourselves...
 	jack_activate(m_pJackClient);
@@ -711,12 +714,12 @@ bool qtractorAudioEngine::activate (void)
 bool qtractorAudioEngine::start (void)
 {
 	if (!isActivated())
-	    return false;
+		return false;
 
-    // Reset all dependables...
-    resetAllMonitors();
+	// Reset all dependables...
+	resetAllMonitors();
 
-    // Make sure we have an actual session cursor...
+	// Make sure we have an actual session cursor...
 	resetMetro();
 
 	// Start transport rolling...
@@ -732,7 +735,7 @@ bool qtractorAudioEngine::start (void)
 void qtractorAudioEngine::stop (void)
 {
 	if (!isActivated())
-	    return;
+		return;
 
 	if (m_transportMode & qtractorBus::Output) {
 		jack_transport_stop(m_pJackClient);
@@ -819,6 +822,16 @@ int qtractorAudioEngine::process ( unsigned int nframes )
 	if (!isActivated())
 		return 0;
 
+	// Reset buffer offset.
+	m_iBufferOffset = 0;
+
+	// Are we actually freewheeling for export?...
+	// notice that freewheeling has no RT requirements.
+	if (m_bFreewheel) {
+		process_export(nframes);
+		return 0;
+	}
+
 	// Must have a valid session...
 	qtractorSession *pSession = session();
 	if (pSession == NULL)
@@ -829,78 +842,6 @@ int qtractorAudioEngine::process ( unsigned int nframes )
 	if (pAudioCursor == NULL)
 		return 0;
 
-	// Reset buffer offset.
-	m_iBufferOffset = 0;
-
-	// Are we actually freewheeling for export?...
-	// notice that freewheeling has no RT requirements.
-	if (m_bFreewheel) {
-		// Make sure we're in a valid state...
-		if (m_pExportFile && m_pExportBuffer && !m_bExportDone) {
-			// This the legal process cycle frame range...
-			const unsigned long iFrameStart = pAudioCursor->frame();
-			const unsigned long iFrameEnd   = iFrameStart + nframes;
-			// Write output bus buffer to export audio file...
-			if (iFrameStart < m_iExportEnd && iFrameEnd > m_iExportStart) {
-				// Force/sync every audio clip approaching...
-				syncExport(iFrameStart, iFrameEnd);
-				// Prepare mix-down buffer...
-				m_pExportBuffer->process_prepare(nframes);
-				// Prepare the output buses first...
-				QListIterator<qtractorAudioBus *> iter(*m_pExportBuses);
-				while (iter.hasNext())
-					iter.next()->process_prepare(nframes);
-				// Prepare all extra audio buses...
-				for (qtractorBus *pBusEx = busesEx().first();
-						pBusEx; pBusEx = pBusEx->next()) {
-					qtractorAudioBus *pAudioBusEx
-						= static_cast<qtractorAudioBus *> (pBusEx);
-					if (pAudioBusEx)
-						pAudioBusEx->process_prepare(nframes);
-				}
-				// Export cycle...
-				pSession->process(pAudioCursor, iFrameStart, iFrameEnd);
-				// Check end-of-export...
-				if (iFrameEnd > m_iExportEnd)
-					nframes -= (iFrameEnd - m_iExportEnd);
-				// Commit the output buses...
-				iter.toFront();
-				while (iter.hasNext()) {
-					qtractorAudioBus *pExportBus = iter.next();
-					pExportBus->process_commit(nframes);
-					m_pExportBuffer->process_add(pExportBus, nframes);
-				}
-				// Write to export file...
-				m_pExportFile->write(m_pExportBuffer->buffer(), nframes);
-				// Prepare advance for next cycle...
-				pAudioCursor->seek(iFrameEnd);
-				// HACK! Freewheeling observers update (non RT safe!)...
-				qtractorSubject::flushQueue(false);
-			} else {
-				// Are we trough?
-				m_bExportDone = true;
-				// HACK: Silence out all audio output buses...
-				for (qtractorBus *pBus = buses().first();
-						pBus; pBus = pBus->next()) {
-					qtractorAudioBus *pAudioBus
-						= static_cast<qtractorAudioBus *> (pBus);
-					if (pAudioBus)
-						pAudioBus->process_prepare(nframes);
-				}
-				// HACK: and all extra audio buses...
-				for (qtractorBus *pBusEx = busesEx().first();
-						pBusEx; pBusEx = pBusEx->next()) {
-					qtractorAudioBus *pAudioBusEx
-						= static_cast<qtractorAudioBus *> (pBusEx);
-					if (pAudioBusEx)
-						pAudioBusEx->process_prepare(nframes);
-				}
-			}
-		}
-		// Done with this one export cycle...
-		return 0;
-	}
-
 	// Session RT-safeness lock...
 	if (!pSession->acquire())
 		return 0;
@@ -1122,6 +1063,124 @@ int qtractorAudioEngine::process ( unsigned int nframes )
 }
 
 
+// Freewheeling process cycle executive (needed for export).
+void qtractorAudioEngine::process_export ( unsigned int nframes )
+{
+	if (m_bExportDone)
+		return;
+	if (m_pExportBuses  == NULL ||
+		m_pExportFile   == NULL ||
+		m_pExportBuffer == NULL)
+		return;
+
+	qtractorSession *pSession = session();
+	if (pSession == NULL)
+		return;
+
+	qtractorSessionCursor *pAudioCursor = sessionCursor();
+	if (pAudioCursor == NULL)
+		return;
+
+	// Make sure we're in a valid state...
+	QListIterator<qtractorAudioBus *> iter(*m_pExportBuses);
+	// Prepare the output buses first...
+	while (iter.hasNext())
+		iter.next()->process_prepare(nframes);
+	// Prepare all extra audio buses...
+	for (qtractorBus *pBusEx = busesEx().first();
+			pBusEx; pBusEx = pBusEx->next()) {
+		qtractorAudioBus *pAudioBusEx
+			= static_cast<qtractorAudioBus *> (pBusEx);
+		if (pAudioBusEx)
+			pAudioBusEx->process_prepare(nframes);
+	}
+
+	// This the legal process cycle frame range...
+	const unsigned long iFrameStart = pAudioCursor->frame();
+	const unsigned long iFrameEnd   = iFrameStart + nframes;
+
+	// Write output bus buffers to export audio file...
+	if (iFrameStart < m_iExportEnd && iFrameEnd > m_iExportStart) {
+		// Prepare mix-down buffer...
+		m_pExportBuffer->process_prepare(nframes);
+		// Force/sync every audio clip approaching...
+	#ifdef CONFIG_LV2
+	#ifdef CONFIG_LV2_TIME
+		qtractorLv2Plugin::updateTime(m_pJackClient);
+	#endif
+	#endif
+		// MIDI plugin manager processing...
+		qtractorMidiManager *pMidiManager
+			= pSession->midiManagers().first();
+		while (pMidiManager) {
+			pMidiManager->process(iFrameStart, iFrameEnd);
+			pMidiManager = pMidiManager->next();
+		}
+		// Perform all tracks processing...
+		int iTrack = 0;
+		for (qtractorTrack *pTrack = pSession->tracks().first();
+				pTrack; pTrack = pTrack->next()) {
+			pTrack->process_export(pAudioCursor->clip(iTrack),
+				iFrameStart, iFrameEnd);
+			++iTrack;
+		}
+		// Prepare advance for next cycle...
+		pAudioCursor->seek(iFrameEnd);
+		// Check end-of-export...
+		if (iFrameEnd > m_iExportEnd)
+			nframes -= (iFrameEnd - m_iExportEnd);
+		// Commit the output buses...
+		iter.toFront();
+		while (iter.hasNext()) {
+			qtractorAudioBus *pExportBus = iter.next();
+			pExportBus->process_commit(nframes);
+			m_pExportBuffer->process_add(pExportBus, nframes);
+		}
+		// Write to export file...
+		m_pExportFile->write(m_pExportBuffer->buffer(), nframes);
+		// HACK! Freewheeling observers update (non RT safe!)...
+		qtractorSubject::flushQueue(false);
+	} else {
+		// Are we trough?
+		m_bExportDone = true;
+		// HACK! Reset all observers...
+		qtractorSubject::resetQueue();
+		// HACK: Reset all MIDI plugin buffers...
+		qtractorMidiManager *pMidiManager
+			= pSession->midiManagers().first();
+		while (pMidiManager) {
+			pMidiManager->reset();
+			pMidiManager = pMidiManager->next();
+		}
+		// HACK: Shut-off (panic) all MIDI tracks...
+		QHash<qtractorMidiBus *, unsigned short> channels;
+		for (qtractorTrack *pTrack = pSession->tracks().first();
+				pTrack; pTrack = pTrack->next()) {
+			if (pTrack->trackType() != qtractorTrack::Midi)
+				continue;
+			qtractorMidiBus *pMidiBus
+				= static_cast<qtractorMidiBus *> (pTrack->outputBus());
+			if (pMidiBus == NULL)
+				continue;
+			const unsigned short iChannel = pTrack->midiChannel();
+			pMidiManager = (pTrack->pluginList())->midiManager();
+			if (pMidiManager)
+				pMidiManager->shutOff(iChannel);
+			const unsigned short iChannelMask = (1 << iChannel);
+			const unsigned short iChannelFlags = channels.value(pMidiBus, 0);
+			if ((iChannelFlags & iChannelMask) == 0) {
+				pMidiManager = (pMidiBus->pluginList_out())->midiManager();
+				if (pMidiManager)
+					pMidiManager->shutOff(iChannel);
+				channels.insert(pMidiBus, iChannelFlags | iChannelMask);
+			}
+		}
+		// HACK: Reset all audio monitors...
+		resetAllMonitors();
+	}
+}
+
+
 // Document element methods.
 bool qtractorAudioEngine::loadElement (
 	qtractorDocument *pDocument, QDomElement *pElement )
@@ -1151,6 +1210,11 @@ bool qtractorAudioEngine::loadElement (
 					qtractorAudioEngine::setTransportMode(
 						qtractorBus::busModeFromText(eProp.text()));
 				}
+				else
+				if (eProp.tagName() == "timebase") {
+					qtractorAudioEngine::setTimebase(
+						qtractorDocument::boolFromText(eProp.text()));
+				}
 			}
 		}
 		else if (eChild.tagName() == "audio-bus") {
@@ -1190,6 +1254,9 @@ bool qtractorAudioEngine::saveElement (
 	pDocument->saveTextElement("transport-mode",
 		qtractorBus::textFromBusMode(
 			qtractorAudioEngine::transportMode()), &eControl);
+	pDocument->saveTextElement("timebase",
+		qtractorDocument::textFromBool(
+			qtractorAudioEngine::isTimebase()), &eControl);
 	pElement->appendChild(eControl);
 
 	// Save audio buses...
@@ -1267,8 +1334,8 @@ bool qtractorAudioEngine::isExporting (void) const
 
 
 // Audio-export method.
-bool qtractorAudioEngine::fileExport ( const QString& sExportPath,
-	const QList<qtractorAudioBus *>& exportBuses,
+bool qtractorAudioEngine::fileExport (
+	const QString& sExportPath, const QList<qtractorAudioBus *>& exportBuses,
 	unsigned long iExportStart, unsigned long iExportEnd )
 {
 	// No simultaneous or foul exports...
@@ -1340,18 +1407,21 @@ bool qtractorAudioEngine::fileExport ( const QString& sExportPath,
 	const unsigned long iPlayHead  = pSession->playHead();
 	const unsigned long iLoopStart = pSession->loopStart();
 	const unsigned long iLoopEnd   = pSession->loopEnd();
-	const bool bMonitor = pExportBus->isMonitor();
+
+	QHash<qtractorAudioBus *, bool> exportMonitors;
+	QListIterator<qtractorAudioBus *> bus_iter(exportBuses);
+	while (bus_iter.hasNext()) {
+		qtractorAudioBus *pAudioBus = bus_iter.next();
+		exportMonitors.insert(pAudioBus, pAudioBus->isMonitor());
+		pAudioBus->setMonitor(false);
+	}
 
 	// Because we'll have to set the export conditions...
 	pSession->setLoop(0, 0);
 	pSession->setPlayHead(m_iExportStart);
-	pExportBus->setMonitor(false);
-
-	// Force initial full-sync...
-	syncExport(m_iExportStart, m_iExportEnd);
 
 	// Special initialization.
-    m_iBufferOffset = 0;
+	m_iBufferOffset = 0;
 
 	// Start export (freewheeling)...
 	jack_set_freewheel(m_pJackClient, 1);
@@ -1371,7 +1441,12 @@ bool qtractorAudioEngine::fileExport ( const QString& sExportPath,
 	// Restore session at ease...
 	pSession->setLoop(iLoopStart, iLoopEnd);
 	pSession->setPlayHead(iPlayHead);
-	pExportBus->setMonitor(bMonitor);
+
+	bus_iter.toFront();
+	while (bus_iter.hasNext()) {
+		qtractorAudioBus *pAudioBus = bus_iter.next();
+		pAudioBus->setMonitor(exportMonitors.value(pAudioBus, false));
+	}
 
 	// Check user cancellation...
 	const bool bResult = m_bExporting;
@@ -1400,38 +1475,6 @@ bool qtractorAudioEngine::fileExport ( const QString& sExportPath,
 }
 
 
-// Direct sync method (needed for export)
-void qtractorAudioEngine::syncExport (
-	unsigned long iFrameStart, unsigned long iFrameEnd )
-{
-	qtractorSession *pSession = session();
-	if (pSession == NULL)
-		return;
-
-	qtractorSessionCursor *pAudioCursor = sessionCursor();
-	if (pAudioCursor == NULL)
-		return;
-
-	int iTrack = 0;
-	for (qtractorTrack *pTrack = pSession->tracks().first();
-			pTrack; pTrack = pTrack->next()) {
-		if (pTrack->trackType() == qtractorTrack::Audio) {
-			qtractorClip *pClip = pAudioCursor->clip(iTrack);
-			while (pClip
-				&& pClip->clipStart() < iFrameEnd
-				&& pClip->clipStart() + pClip->clipLength() > iFrameStart) {
-				qtractorAudioClip *pAudioClip
-					= static_cast<qtractorAudioClip *> (pClip);
-				if (pAudioClip)
-					pAudioClip->syncExport();
-				pClip = pClip->next();
-			}
-		}
-		++iTrack;
-	}
-}
-
-
 // Special track-immediate methods.
 void qtractorAudioEngine::trackMute ( qtractorTrack *pTrack, bool bMute )
 {
@@ -1933,6 +1976,18 @@ qtractorBus::BusMode qtractorAudioEngine::transportMode (void) const
 }
 
 
+// JACK Timebase mode accessors.
+void qtractorAudioEngine::setTimebase ( bool bTimebase )
+{
+	m_bTimebase = bTimebase;
+}
+
+bool qtractorAudioEngine::isTimebase (void) const
+{
+	return m_bTimebase;
+}
+
+
 // Absolute number of frames elapsed since engine start.
 unsigned long qtractorAudioEngine::jackFrame (void) const
 {
@@ -1943,10 +1998,10 @@ unsigned long qtractorAudioEngine::jackFrame (void) const
 // Reset all audio monitoring...
 void qtractorAudioEngine::resetAllMonitors (void)
 {
-    // There must a session reference...
-    qtractorSession *pSession = session();
-    if (pSession == NULL)
-        return;
+	// There must a session reference...
+	qtractorSession *pSession = session();
+	if (pSession == NULL)
+		return;
 
 	// Reset all audio bus monitors...
 	for (qtractorBus *pBus = buses().first();
@@ -1979,9 +2034,9 @@ void qtractorAudioEngine::resetAllMonitors (void)
 //
 
 // Constructor.
-qtractorAudioBus::qtractorAudioBus ( qtractorAudioEngine *pAudioEngine,
-	const QString& sBusName, BusMode busMode, bool bMonitor,
-	unsigned short iChannels )
+qtractorAudioBus::qtractorAudioBus (
+	qtractorAudioEngine *pAudioEngine, const QString& sBusName,
+	BusMode busMode, bool bMonitor, unsigned short iChannels )
 	: qtractorBus(pAudioEngine, sBusName, busMode, bMonitor)
 {
 	m_iChannels = iChannels;
@@ -2100,20 +2155,26 @@ bool qtractorAudioBus::open (void)
 		= static_cast<qtractorAudioEngine *> (engine());
 	if (pAudioEngine == NULL)
 		return false;
-	if (pAudioEngine->jackClient() == NULL)
+
+	jack_client_t *pJackClient = pAudioEngine->jackClient();
+	if (pJackClient == NULL)
 		return false;
 
+	const qtractorBus::BusMode busMode
+		= qtractorAudioBus::busMode();
+	const unsigned int iBufferSize
+		= pAudioEngine->bufferSize();
+
 	unsigned short i;
 	unsigned short iDisabled = 0;
 
-	if (busMode() & qtractorBus::Input) {
+	if (busMode & qtractorBus::Input) {
 		// Register and allocate input port buffers...
 		m_ppIPorts  = new jack_port_t * [m_iChannels];
 		m_ppIBuffer = new float * [m_iChannels];
 		const QString sIPortName(busName() + "/in_%1");
 		for (i = 0; i < m_iChannels; ++i) {
-			m_ppIPorts[i] = jack_port_register(
-				pAudioEngine->jackClient(),
+			m_ppIPorts[i] = jack_port_register(pJackClient,
 				sIPortName.arg(i + 1).toUtf8().constData(),
 				JACK_DEFAULT_AUDIO_TYPE,
 				JackPortIsInput, 0);
@@ -2122,14 +2183,13 @@ bool qtractorAudioBus::open (void)
 		}
 	}
 
-	if (busMode() & qtractorBus::Output) {
+	if (busMode & qtractorBus::Output) {
 		// Register and allocate output port buffers...
 		m_ppOPorts  = new jack_port_t * [m_iChannels];
 		m_ppOBuffer = new float * [m_iChannels];
 		const QString sOPortName(busName() + "/out_%1");
 		for (i = 0; i < m_iChannels; ++i) {
-			m_ppOPorts[i] = jack_port_register(
-				pAudioEngine->jackClient(),
+			m_ppOPorts[i] = jack_port_register(pJackClient,
 				sOPortName.arg(i + 1).toUtf8().constData(),
 				JACK_DEFAULT_AUDIO_TYPE,
 				JackPortIsOutput, 0);
@@ -2139,7 +2199,6 @@ bool qtractorAudioBus::open (void)
 	}
 
 	// Allocate internal working bus buffers...
-	const unsigned int iBufferSize = pAudioEngine->bufferSize();
 	m_ppXBuffer = new float * [m_iChannels];
 	m_ppYBuffer = new float * [m_iChannels];
 	for (i = 0; i < m_iChannels; ++i) {
@@ -2171,47 +2230,50 @@ void qtractorAudioBus::close (void)
 	if (pAudioEngine == NULL)
 		return;
 
+	jack_client_t *pJackClient = pAudioEngine->jackClient();
+
+	const qtractorBus::BusMode busMode
+		= qtractorAudioBus::busMode();
+
 	unsigned short i;
 
-	if (busMode() & qtractorBus::Input) {
-		// Free input ports.
-		if (m_ppIPorts) {
-			// Unregister, if we're not shutdown...
-			if (pAudioEngine->jackClient()) {
-				for (i = 0; i < m_iChannels; ++i) {
-					if (m_ppIPorts[i]) {
-						jack_port_unregister(
-							pAudioEngine->jackClient(), m_ppIPorts[i]);
-						m_ppIPorts[i] = NULL;
-					}
+	if (busMode & qtractorBus::Input) {
+		// Unregister and free input ports,
+		// if we're not shutdown...
+		if (m_ppIPorts && pJackClient) {
+			for (i = 0; i < m_iChannels; ++i) {
+				if (m_ppIPorts[i]) {
+					jack_port_unregister(pJackClient, m_ppIPorts[i]);
+					m_ppIPorts[i] = NULL;
 				}
 			}
-			delete [] m_ppIPorts;
-			m_ppIPorts = NULL;
 		}
-		// Free input Buffers.
+		// Free input ports.
+		if (m_ppIPorts)
+			delete [] m_ppIPorts;
+		m_ppIPorts = NULL;
+		// Free input buffers.
 		if (m_ppIBuffer)
 			delete [] m_ppIBuffer;
 		m_ppIBuffer = NULL;
 	}
 
-	if (busMode() & qtractorBus::Output) {
-		// Free output ports.
-		if (m_ppOPorts) {
-			// Unregister, if we're not shutdown...
-			if (pAudioEngine->jackClient()) {
-				for (i = 0; i < m_iChannels; ++i) {
-					if (m_ppOPorts[i]) {
-						jack_port_unregister(
-							pAudioEngine->jackClient(), m_ppOPorts[i]);
-						m_ppOPorts[i] = NULL;
-					}
+	if (busMode & qtractorBus::Output) {
+		// Unregister and free output ports,
+		// if we're not shutdown...
+		if (m_ppOPorts && pJackClient) {
+			for (i = 0; i < m_iChannels; ++i) {
+				if (m_ppOPorts[i]) {
+					jack_port_unregister(pJackClient, m_ppOPorts[i]);
+					m_ppOPorts[i] = NULL;
 				}
 			}
-			delete [] m_ppOPorts;
-			m_ppOPorts = NULL;
 		}
-		// Free output Buffers.
+		// Free output ports.
+		if (m_ppOPorts)
+			delete [] m_ppOPorts;
+		m_ppOPorts = NULL;
+		// Free output buffers.
 		if (m_ppOBuffer)
 			delete [] m_ppOBuffer;
 		m_ppOBuffer = NULL;
@@ -2235,44 +2297,49 @@ void qtractorAudioBus::close (void)
 // Auto-connect to physical ports.
 void qtractorAudioBus::autoConnect (void)
 {
+	if (!m_bAutoConnect)
+		return;
+
 	qtractorAudioEngine *pAudioEngine
 		= static_cast<qtractorAudioEngine *> (engine());
 	if (pAudioEngine == NULL)
 		return;
-	if (pAudioEngine->jackClient() == NULL)
+
+	jack_client_t *pJackClient = pAudioEngine->jackClient();
+	if (pJackClient == NULL)
 		return;
 
-	if (!m_bAutoConnect)
-		return;
+	const qtractorBus::BusMode busMode
+		= qtractorAudioBus::busMode();
 
 	unsigned short i;
 
-	if ((busMode() & qtractorBus::Input) && inputs().isEmpty()) {
+	if ((busMode & qtractorBus::Input) && inputs().isEmpty()) {
 		const char **ppszOPorts
-			= jack_get_ports(pAudioEngine->jackClient(),
+			= jack_get_ports(pJackClient,
 				0, JACK_DEFAULT_AUDIO_TYPE,
 				JackPortIsOutput | JackPortIsPhysical);
 		if (ppszOPorts) {
 			const QString sIPortName = pAudioEngine->clientName()
 				+ ':' + busName() + "/in_%1";
 			for (i = 0; i < m_iChannels && ppszOPorts[i]; ++i) {
-				jack_connect(pAudioEngine->jackClient(),
+				jack_connect(pJackClient,
 					ppszOPorts[i], sIPortName.arg(i + 1).toUtf8().constData());
 			}
 			::free(ppszOPorts);
 		}
 	}
 
-	if ((busMode() & qtractorBus::Output) && outputs().isEmpty()) {
+	if ((busMode & qtractorBus::Output) && outputs().isEmpty()) {
 		const char **ppszIPorts
-			= jack_get_ports(pAudioEngine->jackClient(),
+			= jack_get_ports(pJackClient,
 				0, JACK_DEFAULT_AUDIO_TYPE,
 				JackPortIsInput | JackPortIsPhysical);
 		if (ppszIPorts) {
 			const QString sOPortName = pAudioEngine->clientName()
 				+ ':' + busName() + "/out_%1";
 			for (i = 0; i < m_iChannels && ppszIPorts[i]; ++i) {
-				jack_connect(pAudioEngine->jackClient(),
+				jack_connect(pJackClient,
 					sOPortName.arg(i + 1).toUtf8().constData(), ppszIPorts[i]);
 			}
 			::free(ppszIPorts);
@@ -2284,10 +2351,11 @@ void qtractorAudioBus::autoConnect (void)
 // Bus mode change event.
 void qtractorAudioBus::updateBusMode (void)
 {
-	const qtractorBus::BusMode mode = busMode();
+	const qtractorBus::BusMode busMode
+		= qtractorAudioBus::busMode();
 
 	// Have a new/old input monitor?
-	if ((mode & qtractorBus::Input) && !(mode & qtractorBus::Ex)) {
+	if ((busMode & qtractorBus::Input) && !(busMode & qtractorBus::Ex)) {
 		if (m_pIAudioMonitor == NULL)
 			m_pIAudioMonitor = new qtractorAudioMonitor(m_iChannels);
 		if (m_pIPluginList == NULL)
@@ -2310,7 +2378,7 @@ void qtractorAudioBus::updateBusMode (void)
 	}
 
 	// Have a new/old output monitor?
-	if ((mode & qtractorBus::Output) && !(mode & qtractorBus::Ex)) {
+	if ((busMode & qtractorBus::Output) && !(busMode & qtractorBus::Ex)) {
 		if (m_pOAudioMonitor == NULL)
 			m_pOAudioMonitor = new qtractorAudioMonitor(m_iChannels);
 		if (m_pOPluginList == NULL)
@@ -2340,16 +2408,19 @@ void qtractorAudioBus::process_prepare ( unsigned int nframes )
 	if (!m_bEnabled)
 		return;
 
+	const qtractorBus::BusMode busMode
+		= qtractorAudioBus::busMode();
+
 	unsigned short i;
 
-	if (busMode() & qtractorBus::Input) {
+	if (busMode & qtractorBus::Input) {
 		for (i = 0; i < m_iChannels; ++i) {
 			m_ppIBuffer[i] = static_cast<float *>
 				(jack_port_get_buffer(m_ppIPorts[i], nframes));
 		}
 	}
 
-	if (busMode() & qtractorBus::Output) {
+	if (busMode & qtractorBus::Output) {
 		for (i = 0; i < m_iChannels; ++i) {
 			m_ppOBuffer[i] = static_cast<float *>
 				(jack_port_get_buffer(m_ppOPorts[i], nframes));
@@ -2366,12 +2437,15 @@ void qtractorAudioBus::process_monitor ( unsigned int nframes )
 	if (!m_bEnabled)
 		return;
 
-	if (busMode() & qtractorBus::Input) {
+	const qtractorBus::BusMode busMode
+		= qtractorAudioBus::busMode();
+
+	if (busMode & qtractorBus::Input) {
 		if (m_pIPluginList && m_pIPluginList->activated())
 			m_pIPluginList->process(m_ppIBuffer, nframes);
 		if (m_pIAudioMonitor)
 			m_pIAudioMonitor->process(m_ppIBuffer, nframes);
-		if (isMonitor() && (busMode() & qtractorBus::Output)) {
+		if (isMonitor() && (busMode & qtractorBus::Output)) {
 			(*m_pfnBufferAdd)(m_ppOBuffer, m_ppIBuffer,
 				nframes, m_iChannels, m_iChannels, 0);
 		}
@@ -2393,8 +2467,8 @@ void qtractorAudioBus::process_commit ( unsigned int nframes )
 
 
 // Bus-buffering methods.
-void qtractorAudioBus::buffer_prepare ( unsigned int nframes,
-	qtractorAudioBus *pInputBus )
+void qtractorAudioBus::buffer_prepare (
+	unsigned int nframes, qtractorAudioBus *pInputBus )
 {
 	if (!m_bEnabled)
 		return;
@@ -2669,20 +2743,23 @@ void qtractorAudioBus::updatePluginList (
 
 // Retrieve all current JACK connections for a given bus mode interface;
 // return the effective number of connection attempts...
-int qtractorAudioBus::updateConnects ( qtractorBus::BusMode busMode,
-	ConnectList& connects, bool bConnect ) const
+int qtractorAudioBus::updateConnects (
+	qtractorBus::BusMode busMode, ConnectList& connects, bool bConnect ) const
 {
+	// Modes must match, at least...
+	if ((busMode & qtractorAudioBus::busMode()) == 0)
+		return 0;
+
+	if (bConnect && connects.isEmpty())
+		return 0;
+
 	qtractorAudioEngine *pAudioEngine
 		= static_cast<qtractorAudioEngine *> (engine());
 	if (pAudioEngine == NULL)
 		return 0;
-	if (pAudioEngine->jackClient() == NULL)
-		return 0;
 
-	// Modes must match, at least...
-	if ((busMode & qtractorAudioBus::busMode()) == 0)
-		return 0;
-	if (bConnect && connects.isEmpty())
+	jack_client_t *pJackClient = pAudioEngine->jackClient();
+	if (pJackClient == NULL)
 		return 0;
 
 	// Which kind of ports?
@@ -2695,8 +2772,8 @@ int qtractorAudioBus::updateConnects ( qtractorBus::BusMode busMode,
 	ConnectItem item;
 	for (item.index = 0; item.index < m_iChannels; ++item.index) {
 		// Get port connections...
-		const char **ppszClientPorts = jack_port_get_all_connections(
-			pAudioEngine->jackClient(), ppPorts[item.index]);
+		const char **ppszClientPorts
+			= jack_port_get_all_connections(pJackClient, ppPorts[item.index]);
 		if (ppszClientPorts) {
 			// Now, for each port...
 			int iClientPort = 0;
@@ -2755,10 +2832,10 @@ int qtractorAudioBus::updateConnects ( qtractorBus::BusMode busMode,
 				sInputPort.toUtf8().constData());
 	#endif
 		// Do it...
-		if (jack_connect(pAudioEngine->jackClient(),
+		if (jack_connect(pJackClient,
 				sOutputPort.toUtf8().constData(),
 				sInputPort.toUtf8().constData()) == 0) {
-			int iItem = connects.indexOf(pItem);
+			const int iItem = connects.indexOf(pItem);
 			if (iItem >= 0) {
 				connects.removeAt(iItem);
 				delete pItem;
@@ -2843,10 +2920,13 @@ bool qtractorAudioBus::loadElement (
 bool qtractorAudioBus::saveElement (
 	qtractorDocument *pDocument, QDomElement *pElement ) const
 {
+	const qtractorBus::BusMode busMode
+		= qtractorAudioBus::busMode();
+
 	pElement->setAttribute("name",
 		qtractorAudioBus::busName());
 	pElement->setAttribute("mode",
-		qtractorBus::textFromBusMode(qtractorAudioBus::busMode()));
+		qtractorBus::textFromBusMode(busMode));
 
 	pDocument->saveTextElement("monitor",
 		qtractorDocument::textFromBool(
@@ -2857,7 +2937,7 @@ bool qtractorAudioBus::saveElement (
 		qtractorDocument::textFromBool(
 			qtractorAudioBus::isAutoConnect()), pElement);
 
-	if (qtractorAudioBus::busMode() & qtractorBus::Input) {
+	if (busMode & qtractorBus::Input) {
 		if (qtractorAudioBus::monitor_in()) {
 			pDocument->saveTextElement("input-gain",
 				QString::number(qtractorAudioBus::monitor_in()->gain()),
@@ -2899,7 +2979,7 @@ bool qtractorAudioBus::saveElement (
 		pElement->appendChild(eAudioInputs);
 	}
 
-	if (qtractorAudioBus::busMode() & qtractorBus::Output) {
+	if (busMode & qtractorBus::Output) {
 		if (qtractorAudioBus::monitor_out()) {
 			pDocument->saveTextElement("output-gain",
 				QString::number(qtractorAudioBus::monitor_out()->gain()),
