@@ -1,7 +1,7 @@
 // qtractorAudioEngine.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -147,9 +147,6 @@ public:
 		const QList<qtractorAudioBus *>& exportBuses,
 		unsigned long iExportStart = 0, unsigned long iExportEnd = 0);
 
-	// Direct sync method (needed for export)
-	void syncExport(unsigned long iFrameStart, unsigned long iFrameEnd);
-
 	// Special track-immediate methods.
 	void trackMute(qtractorTrack *pTrack, bool bMute);
 
@@ -210,6 +207,10 @@ public:
 	void setTransportMode(qtractorBus::BusMode transportMode);
 	qtractorBus::BusMode transportMode() const;
 
+	// JACK Timebase mode accessors.
+	void setTimebase(bool bTimebase);
+	bool isTimebase() const;
+
 	// Absolute number of frames elapsed since engine start.
 	unsigned long jackFrame() const;
 
@@ -237,6 +238,9 @@ protected:
 	void closePlayerBus();
 	void deletePlayerBus();
 
+	// Freewheeling process cycle executive (needed for export).
+	void process_export(unsigned int nframes);
+
 private:
 
 	// Special event notifier proxy object.
@@ -301,6 +305,9 @@ private:
 
 	// JACK Transport mode.
 	qtractorBus::BusMode m_transportMode;
+
+	// JACK Timebase mode.
+	bool m_bTimebase;
 };
 
 
