@@ -1,7 +1,7 @@
 // qtractorClipCommand.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -99,12 +99,12 @@ void qtractorClipCommand::fileClip ( qtractorClip *pClip,
 	pItem->trackChannel = iTrackChannel;
 	m_items.append(pItem);
 
-	reopenClip(pClip, true);
+	reopenClip(pClip, (pClip->track())->trackType() == qtractorTrack::Audio);
 }
 
 
-void qtractorClipCommand::renameClip ( qtractorClip *pClip,
-	const QString& sClipName )
+void qtractorClipCommand::renameClip (
+	qtractorClip *pClip, const QString& sClipName )
 {
 	Item *pItem = new Item(RenameClip, pClip, pClip->track());
 	pItem->clipName = sClipName;
@@ -180,7 +180,7 @@ void qtractorClipCommand::resizeClip ( qtractorClip *pClip,
 		case qtractorTrack::Audio: {
 			qtractorAudioClip *pAudioClip
 				= static_cast<qtractorAudioClip *> (pClip);
-			float fRatio = fTimeStretch / pAudioClip->timeStretch();
+			const float fRatio = fTimeStretch / pAudioClip->timeStretch();
 			pItem->clipOffset = (unsigned long) (fRatio * float(iClipOffset));
 			break;
 		}
@@ -234,8 +234,8 @@ void qtractorClipCommand::fadeOutClip ( qtractorClip *pClip,
 }
 
 
-void qtractorClipCommand::timeStretchClip ( qtractorClip *pClip,
-	float fTimeStretch )
+void qtractorClipCommand::timeStretchClip (
+	qtractorClip *pClip, float fTimeStretch )
 {
 	Item *pItem = new Item(TimeStretchClip, pClip, pClip->track());
 	pItem->timeStretch = fTimeStretch;
@@ -245,8 +245,8 @@ void qtractorClipCommand::timeStretchClip ( qtractorClip *pClip,
 }
 
 
-void qtractorClipCommand::pitchShiftClip ( qtractorClip *pClip,
-	float fPitchShift )
+void qtractorClipCommand::pitchShiftClip (
+	qtractorClip *pClip, float fPitchShift )
 {
 	Item *pItem = new Item(PitchShiftClip, pClip, pClip->track());
 	pItem->pitchShift = fPitchShift;
@@ -256,8 +256,8 @@ void qtractorClipCommand::pitchShiftClip ( qtractorClip *pClip,
 }
 
 
-void qtractorClipCommand::takeInfoClip ( qtractorClip *pClip,
-	qtractorClip::TakeInfo *pTakeInfo )
+void qtractorClipCommand::takeInfoClip (
+	qtractorClip *pClip, qtractorClip::TakeInfo *pTakeInfo )
 {
 	Item *pItem = new Item(TakeInfoClip, pClip, pClip->track());
 	pItem->takeInfo = pTakeInfo;
@@ -294,15 +294,74 @@ void qtractorClipCommand::reopenClip ( qtractorClip *pClip, bool bClose )
 bool qtractorClipCommand::addClipRecord (
 	qtractorTrack *pTrack, unsigned long iFrameTime )
 {
+	qtractorSession *pSession = pTrack->session();
+	if (pSession == NULL)
+		return false;
+
 	qtractorClip *pClip = pTrack->clipRecord();
 	if (pClip == NULL)
 		return false;
 
+	// Recording clip extents...
 	const unsigned long iClipEnd = pTrack->clipRecordEnd(iFrameTime);
 	const unsigned long iClipStart = pClip->clipStart();
 	if (iClipStart >= iClipEnd)
 		return false;
 
+	// Recorded clip length and offset...
+	const unsigned long iClipLength = iClipEnd - iClipStart;
+	const unsigned long iClipOffset = pClip->clipOffset();
+
+	// HACK: Exclusive MIDI clip recording/overdub...
+	if (pTrack->isClipRecordEx()) {
+		if (pTrack->trackType() == qtractorTrack::Midi) {
+			qtractorMidiClip *pMidiClip
+				= static_cast<qtractorMidiClip *> (pClip);
+			if (pMidiClip) {
+				// HACK: Make all final adjustments...
+				qtractorMidiEditCommand(pMidiClip, name()).adjust();
+				// Have a new filename revision...
+				const QString& sFilename
+					= pMidiClip->createFilePathRevision(true);
+				// Save/replace the overdubbed clip...
+				qtractorMidiFile::saveCopyFile(
+					sFilename,
+					pMidiClip->filename(),
+					pMidiClip->trackChannel(),
+					pMidiClip->format(),
+					pMidiClip->sequence(),
+					pSession->timeScale(),
+					pSession->tickFromFrame(iClipStart));
+			#if 0
+				// Original clip length stays put...
+				const unsigned long iClipLengthEx = pMidiClip->clipLength();
+				// Remove original clip anyway...
+				removeClip(pMidiClip);
+				// Clone into a new one...
+				pMidiClip = new qtractorMidiClip(*pMidiClip);
+				pMidiClip->setClipStart(iClipStart);
+				pMidiClip->setClipOffset(iClipOffset);
+				pMidiClip->setClipLength(iClipLengthEx);
+				pMidiClip->setFilename(sFilename);
+				pMidiClip->setDirty(true);
+				addClip(pMidiClip, pTrack);
+			#else
+				// Just change filename/track-channel...
+				fileClip(pMidiClip, sFilename, pMidiClip->trackChannel());
+			#endif
+				// Post-commit dirty changes...
+				pSession->files()->addClipItem(qtractorFileList::Midi, sFilename, true);
+				// Add the new file version to the roster...
+				qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
+				if (pMainForm)
+					pMainForm->addMidiFile(sFilename);
+			}
+		}
+		// Can get rid of the recorded clip.
+		pTrack->setClipRecord(NULL);
+		return true;
+	}
+
 	// Time to close the clip...
 	pClip->close();
 
@@ -310,14 +369,8 @@ bool qtractorClipCommand::addClipRecord (
 	if (pClip->clipLength() < 1)
 		return false;
 
-	// Recorded clip length and offset...
-	const unsigned long iClipLength = iClipEnd - iClipStart;
-	const unsigned long iClipOffset = pClip->clipOffset();
-
 	// Check whether in loop-recording/takes mode....
-	qtractorSession *pSession = qtractorSession::getInstance();
-	if (pSession && pSession->isLoopRecording()
-		&& iClipEnd > pSession->loopEnd()) {
+	if (pSession->isLoopRecording() && iClipEnd > pSession->loopEnd()) {
 		// HACK: Take care of punch-in/out...
 		unsigned long iTakeStart = pSession->loopStart();
 		unsigned long iTakeEnd = pSession->loopEnd();
@@ -363,7 +416,7 @@ bool qtractorClipCommand::addClipRecordTake ( qtractorTrack *pTrack,
 
 	// Now, its imperative to make a proper copy of those clips...
 	// -- formerly a cloning clip factory method.
-	
+
 	// Reference for immediate file addition...
 	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
 
@@ -529,7 +582,7 @@ bool qtractorClipCommand::execute ( bool bRedo )
 			break;
 		}
 		case FileClip: {
-			QString sOldFilename = pClip->filename();
+			const QString sOldFilename = pClip->filename();
 			unsigned short iOldTrackChannel = 0;
 			pClip->setFilename(pItem->filename);
 			qtractorMidiClip *pMidiClip = NULL;
@@ -539,16 +592,14 @@ bool qtractorClipCommand::execute ( bool bRedo )
 				iOldTrackChannel = pMidiClip->trackChannel();
 				pMidiClip->setTrackChannel(pItem->trackChannel);
 			}
-		//--pClip->close();	// Scrap peak file (audio).
-		//--pClip->open();
 			pItem->filename = sOldFilename;
 			if (pMidiClip)
 				pItem->trackChannel = iOldTrackChannel;
-			pSession->updateTrack(pTrack);
+			//pSession->updateTrack(pTrack);
 			break;
 		}
 		case RenameClip: {
-			QString sOldName = pClip->clipName();
+			const QString sOldName = pClip->clipName();
 			pClip->setClipName(pItem->clipName);
 			pItem->clipName = sOldName;
 			break;
@@ -598,9 +649,6 @@ bool qtractorClipCommand::execute ( bool bRedo )
 						fOldPitchShift = pAudioClip->pitchShift();
 				}
 			}
-		//--else
-		//--if (pItem->editCommand == NULL)
-		//--	pClip->close();
 			if (iOldStart != pItem->clipStart)
 				pTrack->unlinkClip(pClip);
 			pClip->setClipStart(pItem->clipStart);
@@ -624,7 +672,6 @@ bool qtractorClipCommand::execute ( bool bRedo )
 				else
 					(pItem->editCommand)->undo();
 			}
-		//--else pClip->open();
 			if (iOldStart != pItem->clipStart)
 				pTrack->insertClip(pClip);
 			pItem->clipStart  = iOldStart;
@@ -666,9 +713,7 @@ bool qtractorClipCommand::execute ( bool bRedo )
 			if (pAudioClip) {
 				const float fOldTimeStretch = pAudioClip->timeStretch();
 				pAudioClip->setTimeStretch(pItem->timeStretch);
-			//--pAudioClip->close();			// Scrap peak file.
 				pAudioClip->updateClipTime();	// Care of tempo change.
-			//--pAudioClip->open();
 				pItem->timeStretch = fOldTimeStretch;
 			}
 			break;
@@ -680,7 +725,6 @@ bool qtractorClipCommand::execute ( bool bRedo )
 			if (pAudioClip) {
 				const float fOldPitchShift = pAudioClip->pitchShift();
 				pAudioClip->setPitchShift(pItem->pitchShift);
-			//--pAudioClip->open();
 				pItem->pitchShift = fOldPitchShift;
 			}
 			break;
@@ -1028,4 +1072,49 @@ bool qtractorClipToolCommand::undo (void)
 }
 
 
+//----------------------------------------------------------------------
+// class qtractorClipRecordExCommand - implementation.
+//
+
+// Constructor.
+qtractorClipRecordExCommand::qtractorClipRecordExCommand (
+	qtractorClip *pClipRecordEx, bool bClipRecordEx )
+	: qtractorCommand(QObject::tr("clip record"))
+{
+	m_pTrack = pClipRecordEx->track();
+
+	m_pClipRecordEx = pClipRecordEx;
+	m_bClipRecordEx = bClipRecordEx;
+
+	setRefresh(true);
+}
+
+
+// Clip-record command method.
+bool qtractorClipRecordExCommand::redo (void)
+{
+	if (m_pTrack == NULL)
+		return false;
+
+	// Carry on...
+	qtractorClip *pClipRecordEx = m_pTrack->clipRecord();
+	const bool bClipRecordEx = m_pTrack->isClipRecordEx();
+
+	m_pTrack->setRecord(m_bClipRecordEx);
+	m_pTrack->setClipRecord(m_bClipRecordEx ? m_pClipRecordEx : NULL);
+	m_pTrack->setClipRecordEx(m_bClipRecordEx);
+
+	// Reset for undo.
+	m_bClipRecordEx = bClipRecordEx;
+	m_pClipRecordEx = pClipRecordEx;
+
+	return true;
+}
+
+bool qtractorClipRecordExCommand::undo (void)
+{
+	return redo();
+}
+
+
 // end of qtractorClipCommand.cpp
