@@ -1,7 +1,7 @@
 // qtractorMainForm.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -191,6 +191,7 @@ public slots:
 	void clipNew();
 	void clipEdit();
 	void clipUnlink();
+	void clipRecordEx(bool bOn);
 	void clipSplit();
 	void clipMerge();
 	void clipNormalize();
@@ -391,6 +392,7 @@ protected:
 	void updateDisplayFormat();
 	void updatePluginPaths();
 	void updateTransportMode();
+	void updateTimebase();
 	void updateMidiControlModes();
 	void updateAudioPlayer();
 	void updateMidiQueueTimer();
@@ -455,6 +457,7 @@ private:
 	int m_iPeakTimer;
 	int m_iPlayTimer;
 	int m_iIdleTimer;
+	int m_iTransportTimer;
 	int m_iTransportUpdate;
 	int m_iTransportRolling;
 	bool m_bTransportPlaying;
