@@ -1,7 +1,7 @@
 // qtractorMidiClip.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -28,11 +28,15 @@
 
 #include "qtractorDocument.h"
 
+#include "qtractorMidiBuffer.h"
+#include "qtractorPlugin.h"
+
 #include "qtractorMidiEditor.h"
 #include "qtractorMidiEditorForm.h"
 
 #include "qtractorMainForm.h"
 
+
 #ifdef QTRACTOR_MIDI_EDITOR_TOOL
 #include "qtractorOptions.h"
 #endif
@@ -341,7 +345,6 @@ bool qtractorMidiClip::openMidiFile (
 
 	// Set local properties...
 	setFilename(sFilename);
-	setTrackChannel(iTrackChannel);
 	setDirty(false);
 
 	// Register file path...
@@ -410,15 +413,12 @@ bool qtractorMidiClip::openMidiFile (
 	if (bWrite) {
 		// On write mode, iTrackChannel holds the SMF format,
 		// so we'll convert it here as properly.
-		unsigned short iFormat = 0;
+		const unsigned short iFormat
+			= qtractorMidiClip::defaultFormat();
 		unsigned short iTracks = 1;
-		if (iTrackChannel == 0) {
-			// SMF format 0 (1 track, 1 channel)
-			iTrackChannel = pTrack->midiChannel();
-		} else {
+		if (iFormat == 1) {
 			// SMF format 1 (2 tracks, 1 channel)
 			iTrackChannel = 1;
-			iFormat = 1;
 			++iTracks;
 		}
 		// That's it.
@@ -469,8 +469,9 @@ bool qtractorMidiClip::openMidiFile (
 				pMidiBus->importSysexList(pSeq);
 		#endif
 			// Import tempo map as well...
-			if (m_pFile->tempoMap()) {
-				m_pFile->tempoMap()->intoTimeScale(pSession->timeScale(), t0);
+			qtractorMidiFileTempo *pTempoMap = m_pFile->tempoMap();
+			if (pTempoMap) {
+				pTempoMap->intoTimeScale(pSession->timeScale(), t0);
 				pSession->updateTimeScale();
 			}
 			// Reset session flag now.
@@ -481,6 +482,9 @@ bool qtractorMidiClip::openMidiFile (
 		//	return false;
 	}
 
+	// Actual track-channel is set by now...
+	setTrackChannel(iTrackChannel);
+
 	// Make it a brand new revision...
 	// setRevision(1);
 
@@ -504,6 +508,10 @@ bool qtractorMidiClip::openMidiFile (
 	updateHashKey();
 	insertHashKey();
 
+	// Update/reset MIDI clip editor if any...
+	if (m_pMidiEditorForm)
+		m_pMidiEditorForm->setup(this);
+
 	return true;
 }
 
@@ -511,12 +519,6 @@ bool qtractorMidiClip::openMidiFile (
 // Private cleanup.
 void qtractorMidiClip::closeMidiFile (void)
 {
-	if (m_pMidiEditorForm) {
-		m_pMidiEditorForm->close();
-		delete m_pMidiEditorForm;
-		m_pMidiEditorForm = NULL;
-	}
-
 	if (m_pData) {
 		m_pData->detach(this);
 		if (m_pData->count() < 1) {
@@ -528,7 +530,6 @@ void qtractorMidiClip::closeMidiFile (void)
 		qtractorSession *pSession = qtractorSession::getInstance();
 		if (pSession)
 			pSession->files()->removeClipItem(qtractorFileList::Midi, this);
-
 	}
 
 	if (m_pKey) {
@@ -870,6 +871,13 @@ void qtractorMidiClip::close (void)
 		m_pFile->close();
 	}
 
+	// Sure close MIDI clip editor if any...
+	if (m_pMidiEditorForm) {
+		m_pMidiEditorForm->close();
+		delete m_pMidiEditorForm;
+		m_pMidiEditorForm = NULL;
+	}
+
 	// Just to be sure things get deallocated..
 	closeMidiFile();
 
@@ -937,6 +945,48 @@ void qtractorMidiClip::process (
 }
 
 
+// MIDI clip freewheeling process cycle executive (needed for export).
+void qtractorMidiClip::process_export (
+	unsigned long iFrameStart, unsigned long iFrameEnd )
+{
+	qtractorTrack *pTrack = track();
+	if (pTrack == NULL)
+		return;
+
+	qtractorSession *pSession = pTrack->session();
+	if (pSession == NULL)
+		return;
+
+	qtractorMidiSequence *pSeq = sequence();
+	if (pSeq == NULL)
+		return;
+
+	// Track mute state...
+	const bool bMute = (pTrack->isMute()
+		|| (pSession->soloTracks() && !pTrack->isSolo()));
+
+	const unsigned long t0 = pSession->tickFromFrame(clipStart());
+
+	const unsigned long iTimeStart = pSession->tickFromFrame(iFrameStart);
+	const unsigned long iTimeEnd   = pSession->tickFromFrame(iFrameEnd);
+
+	// Enqueue the requested events...
+	qtractorMidiEvent *pEvent
+		= m_playCursor.seek(pSeq, iTimeStart > t0 ? iTimeStart - t0 : 0);
+	while (pEvent) {
+		const unsigned long t1 = t0 + pEvent->time();
+		if (t1 >= iTimeEnd)
+			break;
+		if (t1 >= iTimeStart
+			&& (!bMute || pEvent->type() != qtractorMidiEvent::NOTEON)) {
+			enqueue_export(pTrack, pEvent, t1,
+				gain(pSession->frameFromTick(t1) - clipStart()));
+		}
+		pEvent = pEvent->next();
+	}
+}
+
+
 // MIDI clip paint method.
 void qtractorMidiClip::draw (
 	QPainter *pPainter, const QRect& clipRect, unsigned long iClipOffset )
@@ -953,10 +1003,15 @@ void qtractorMidiClip::draw (
 	if (pSeq == NULL)
 		return;
 
-	// Check maximum note span...
-	int iNoteSpan = (pSeq->noteMax() - pSeq->noteMin()) + 1;
-	if (iNoteSpan < 6)
+	// Check min/maximum note span...
+	int iNoteMin  = pSeq->noteMin() - 1;
+	int iNoteSpan = pSeq->noteMax() - iNoteMin;
+	if (iNoteSpan < 6) {
 		iNoteSpan = 6;
+		--iNoteMin;
+	}
+	if (iNoteMin < 0)
+		iNoteMin = 0;
 
 	const unsigned long iClipStart = clipStart();
 	const unsigned long iFrameStart = iClipStart + iClipOffset;
@@ -996,7 +1051,7 @@ void qtractorMidiClip::draw (
 				pNode = cursor.seekTick(t1);
 				const int x = clipRect.x() + pNode->pixelFromTick(t1) - cx;
 				const int y = clipRect.bottom()
-					- (h1 * (pEvent->note() - pSeq->noteMin() + 1)) / iNoteSpan;
+					- (h1 * (pEvent->note() - iNoteMin)) / iNoteSpan;
 				pNode = cursor.seekTick(t2);
 				int w = (t1 < t2 || !bClipRecord
 					? clipRect.x() + pNode->pixelFromTick(t2) - cx
@@ -1039,6 +1094,10 @@ bool qtractorMidiClip::startEditor ( QWidget *pParent )
 		// Do it...
 		m_pMidiEditorForm = new qtractorMidiEditorForm(pParent, wflags);
 		// Set its most standing properties...
+		if (!m_posEditor.isNull())
+			m_pMidiEditorForm->move(m_posEditor);
+		if (!m_sizeEditor.isNull() && m_sizeEditor.isValid())
+			m_pMidiEditorForm->resize(m_sizeEditor);
 		m_pMidiEditorForm->show();
 		m_pMidiEditorForm->setup(this);
 	} else {
@@ -1092,6 +1151,18 @@ void qtractorMidiClip::updateEditor ( bool bSelectClear )
 }
 
 
+// Clip editor update.
+void qtractorMidiClip::updateEditorContents (void)
+{
+	if (m_pMidiEditorForm == NULL)
+		return;
+
+	qtractorMidiEditor *pMidiEditor = m_pMidiEditorForm->editor();
+	if (pMidiEditor)
+		pMidiEditor->updateContents();
+}
+
+
 // Clip query-close method (return true if editing is done).
 bool qtractorMidiClip::queryEditor (void)
 {
@@ -1192,6 +1263,16 @@ bool qtractorMidiClip::loadClipElement (
 			qtractorMidiClip::setTrackChannel(eChild.text().toUShort());
 		else if (eChild.tagName() == "revision")
 			qtractorMidiClip::setRevision(eChild.text().toUShort());
+		else if (eChild.tagName() == "editor-pos") {
+			const QStringList& sxy = eChild.text().split(',');
+			m_posEditor.setX(sxy.at(0).toInt());
+			m_posEditor.setY(sxy.at(1).toInt());
+		}
+		else if (eChild.tagName() == "editor-size") {
+			const QStringList& swh = eChild.text().split(',');
+			m_sizeEditor.setWidth(swh.at(0).toInt());
+			m_sizeEditor.setHeight(swh.at(1).toInt());
+		}
 	}
 
 	return true;
@@ -1199,8 +1280,14 @@ bool qtractorMidiClip::loadClipElement (
 
 
 bool qtractorMidiClip::saveClipElement (
-	qtractorDocument *pDocument, QDomElement *pElement ) const
+	qtractorDocument *pDocument, QDomElement *pElement )
 {
+	// Freeze current MIDI clip editor, if up and visible...
+	if (m_pMidiEditorForm && m_pMidiEditorForm->isVisible()) {
+		m_posEditor = m_pMidiEditorForm->pos();
+		m_sizeEditor = m_pMidiEditorForm->size();
+	}
+
 	QDomElement eMidiClip = pDocument->document()->createElement("midi-clip");
 	pDocument->saveTextElement("filename",
 		qtractorMidiClip::relativeFilename(pDocument), &eMidiClip);
@@ -1208,6 +1295,16 @@ bool qtractorMidiClip::saveClipElement (
 		QString::number(qtractorMidiClip::trackChannel()), &eMidiClip);
 	pDocument->saveTextElement("revision",
 		QString::number(qtractorMidiClip::revision()), &eMidiClip);
+	if (!m_posEditor.isNull()) {
+		pDocument->saveTextElement("editor-pos",
+			QString::number(m_posEditor.x()) + ',' +
+			QString::number(m_posEditor.y()), &eMidiClip);
+	}
+	if (!m_sizeEditor.isNull() && m_sizeEditor.isValid()) {
+		pDocument->saveTextElement("editor-size",
+			QString::number(m_sizeEditor.width()) + ',' +
+			QString::number(m_sizeEditor.height()), &eMidiClip);
+	}
 	pElement->appendChild(eMidiClip);
 
 	return true;
@@ -1276,6 +1373,92 @@ bool qtractorMidiClip::clipExport (
 }
 
 
+// MIDI clip freewheeling event enqueue method (needed for export).
+void qtractorMidiClip::enqueue_export ( qtractorTrack *pTrack,
+	qtractorMidiEvent *pEvent, unsigned long iTime, float fGain ) const
+{
+	qtractorSession *pSession = pTrack->session();
+	if (pSession == NULL)
+		return;
+
+	snd_seq_event_t ev;
+	snd_seq_ev_clear(&ev);
+
+	snd_seq_ev_schedule_tick(&ev, 0, 0, iTime);
+
+	switch (pEvent->type()) {
+	case qtractorMidiEvent::NOTEON:
+		ev.type = SND_SEQ_EVENT_NOTE;
+		ev.data.note.channel  = pTrack->midiChannel();
+		ev.data.note.note     = pEvent->note();
+		ev.data.note.velocity = int(fGain * float(pEvent->value())) & 0x7f;
+		ev.data.note.duration = pEvent->duration();
+		break;
+	case qtractorMidiEvent::KEYPRESS:
+		ev.type = SND_SEQ_EVENT_KEYPRESS;
+		ev.data.note.channel  = pTrack->midiChannel();
+		ev.data.note.note     = pEvent->note();
+		ev.data.note.velocity = pEvent->velocity();
+		ev.data.note.duration = 0;
+		break;
+	case qtractorMidiEvent::CONTROLLER:
+		ev.type = SND_SEQ_EVENT_CONTROLLER;
+		ev.data.control.channel = pTrack->midiChannel();
+		ev.data.control.param   = pEvent->controller();
+		ev.data.control.value   = pEvent->value();
+		break;
+	case qtractorMidiEvent::PGMCHANGE:
+		ev.type = SND_SEQ_EVENT_PGMCHANGE;
+		ev.data.control.channel = pTrack->midiChannel();
+		ev.data.control.value   = pEvent->value();
+		break;
+	case qtractorMidiEvent::CHANPRESS:
+		ev.type = SND_SEQ_EVENT_CHANPRESS;
+		ev.data.control.channel = pTrack->midiChannel();
+		ev.data.control.value   = pEvent->value();
+		break;
+	case qtractorMidiEvent::PITCHBEND:
+		ev.type = SND_SEQ_EVENT_PITCHBEND;
+		ev.data.control.channel = pTrack->midiChannel();
+		ev.data.control.value   = pEvent->pitchBend();
+		break;
+	case qtractorMidiEvent::SYSEX:
+		ev.type = SND_SEQ_EVENT_SYSEX;
+		snd_seq_ev_set_sysex(&ev, pEvent->sysex_len(), pEvent->sysex());
+		break;
+	default:
+		break;
+	}
+
+	qtractorTimeScale::Cursor& cursor = pSession->timeScale()->cursor();
+	qtractorTimeScale::Node *pNode = cursor.seekTick(iTime);
+	const unsigned long t1 = pNode->frameFromTick(iTime);
+	unsigned long t2 = t1;
+	if (ev.type == SND_SEQ_EVENT_NOTE
+		&& ev.data.note.duration > 0) {
+		iTime += (ev.data.note.duration - 1);
+		pNode = cursor.seekTick(iTime);
+		t2 += (pNode->frameFromTick(iTime) - t1);
+	}
+
+	// Do it for the MIDI track plugins...
+	qtractorMidiManager *pMidiManager
+		= (pTrack->pluginList())->midiManager();
+	if (pMidiManager)
+		pMidiManager->queued(&ev, t1, t2);
+
+	// And for the MIDI output plugins as well...
+	// Target MIDI bus...
+	qtractorMidiBus *pMidiBus
+		= static_cast<qtractorMidiBus *> (pTrack->outputBus());
+	if (pMidiBus && pMidiBus->pluginList_out()) {
+		pMidiManager = (pMidiBus->pluginList_out())->midiManager();
+		if (pMidiManager)
+			pMidiManager->queued(&ev, t1, t2);
+	}
+}
+
+
 // Default MIDI file format (for capture/record) accessors.
 unsigned short qtractorMidiClip::g_iDefaultFormat = 0;
 
