@@ -1,7 +1,7 @@
 // qtractorMidiClip.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -114,6 +114,9 @@ public:
 	// MIDI clip special process cycle executive.
 	void process(unsigned long iFrameStart, unsigned long iFrameEnd);
 
+	// MIDI clip freewheeling process cycle executive (needed for export).
+	void process_export(unsigned long iFrameStart, unsigned long iFrameEnd);
+
 	// Clip paint method.
 	void draw(QPainter *pPainter, const QRect& clipRect,
 		unsigned long iClipOffset);
@@ -122,6 +125,7 @@ public:
 	bool startEditor(QWidget *pParent);
 	void resetEditor(bool bSelectClear);
 	void updateEditor(bool bSelectClear);
+	void updateEditorContents();
 	bool queryEditor();
 
 	// MIDI clip tool-tip.
@@ -231,15 +235,30 @@ public:
 	// Make sure the clip hash-table gets reset.
 	static void clearHashTable();
 
+	// MIDI clip editor position/size accessors.
+	void setEditorPos(const QPoint& pos)
+		{ m_posEditor = pos; }
+	const QPoint& editorPos() const
+		{ return m_posEditor; }
+
+	void setEditorSize(const QSize& size)
+		{ m_sizeEditor = size; }
+	const QSize& editorSize() const
+		{ return m_sizeEditor; }
+
 protected:
 
 	// Virtual document element methods.
 	bool loadClipElement(qtractorDocument *pDocument, QDomElement *pElement);
-	bool saveClipElement(qtractorDocument *pDocument, QDomElement *pElement) const;
+	bool saveClipElement(qtractorDocument *pDocument, QDomElement *pElement);
 
 	// Private cleanup.
 	void closeMidiFile();
 
+	// MIDI clip freewheeling event enqueue method (needed for export).
+	void enqueue_export(qtractorTrack *pTrack,
+		qtractorMidiEvent *pEvent, unsigned long iTime, float fGain) const;
+
 private:
 
 	// Instance variables.
@@ -273,6 +292,10 @@ private:
 	// This clip editor form widget.
 	qtractorMidiEditorForm *m_pMidiEditorForm;
 
+	// And for geometry it was last seen...
+	QPoint m_posEditor;
+	QSize m_sizeEditor;
+
 	// Default MIDI file format (for capture/record)
 	static unsigned short g_iDefaultFormat;
 };
