@@ -1,7 +1,7 @@
 // qtractorMidiEditView.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -289,9 +289,10 @@ void qtractorMidiEditView::updatePixmap ( int cx, int cy )
 
 	// Account for the editing offset:
 	qtractorTimeScale::Cursor cursor(pTimeScale);
-	qtractorTimeScale::Node *pNode = cursor.seekFrame(m_pEditor->offset());
-	const unsigned long t0 = pNode->tickFromFrame(m_pEditor->offset());
-	const int x0 = pTimeScale->pixelFromFrame(m_pEditor->offset());
+	const unsigned long f0 = m_pEditor->offset();
+	qtractorTimeScale::Node *pNode = cursor.seekFrame(f0);
+	const unsigned long t0 = pNode->tickFromFrame(f0);
+	const int x0 = pTimeScale->pixelFromFrame(f0);
 	const int dx = x0 + cx;
 
 	// Draw vertical grid lines...
@@ -365,6 +366,10 @@ void qtractorMidiEditView::updatePixmap ( int cx, int cy )
 	pNode = cursor.seekPixel(x += w);
 	const unsigned long iTickEnd = pNode->tickFromPixel(x);
 
+	const unsigned long f1 = f0 + m_pEditor->length();
+	pNode = cursor.seekFrame(f1);
+	const unsigned long iTimeEnd = pNode->tickFromFrame(f1);
+
 //	p.setPen(rgbFore);
 //	p.setBrush(rgbBack);
 
@@ -378,7 +383,9 @@ void qtractorMidiEditView::updatePixmap ( int cx, int cy )
 		const unsigned long t1 = t0 + pEvent->time();
 		if (t1 >= iTickEnd)
 			break;
-		const unsigned long t2 = t1 + pEvent->duration();
+		unsigned long t2 = t1 + pEvent->duration();
+		if (t2 > iTimeEnd)
+			t2 = iTimeEnd;
 		// Filter event type!...
 		if (pEvent->type() == m_eventType && t2 >= iTickStart) {
 			y = ch - h1 * (pEvent->note() + 1);
@@ -386,7 +393,9 @@ void qtractorMidiEditView::updatePixmap ( int cx, int cy )
 				pNode = cursor.seekTick(t1);
 				x = pNode->pixelFromTick(t1) - dx;
 				pNode = cursor.seekTick(t2);
-				int w1 = pNode->pixelFromTick(t2) - dx - x;
+				int w1 = (t1 >= t2 && m_pEditor->isClipRecord()
+					? m_pEditor->playHeadX()
+					: pNode->pixelFromTick(t2) - dx) - x;
 				if (w1 < 5) w1 = 5;
 				if (m_pEditor->isNoteColor()) {
 					hue = (128 - int(pEvent->note())) << 4;
