@@ -1,7 +1,7 @@
 // qtractorMidiEditor.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -142,6 +142,9 @@ public:
 	void setLength(unsigned long iLength);
 	unsigned long length() const;
 
+	// Clip recording/overdub status.
+	bool isClipRecord() const;
+
 	// Child widgets accessors.
 	QFrame *editListHeader() const;
 	qtractorMidiEditList *editList() const;
@@ -198,10 +201,6 @@ public:
 	void setSnapToScaleType(int iSnapToScaleType);
 	int snapToScaleType() const;
 
-	// Alterrnate command action update helper...
-	void updateUndoAction(QAction *pAction) const;
-	void updateRedoAction(QAction *pAction) const;
-
 	// Command predicate status.
 	bool canUndo() const;
 	bool canRedo() const;
@@ -327,6 +326,9 @@ public:
 	void setSyncViewHold(bool bSyncViewHold);
 	bool isSyncViewHold() const;
 
+	// Make given frame position visible in view.
+	void ensureVisibleFrame(qtractorScrollView *pScrollView, unsigned long iFrame);
+
 	// Visualize the event selection drag-move.
 	void paintDragState(qtractorScrollView *pScrollView,
 		QPainter *pPainter);
@@ -429,7 +431,7 @@ protected:
 	void zoomCenterPre(ZoomCenter& zc) const;
 	void zoomCenterPost(const ZoomCenter& zc);
 
-	// Ensuere point visibility depending on view.
+	// Ensure point visibility depending on view.
 	void ensureVisible(qtractorScrollView *pScrollView, const QPoint& pos);
 
 	// Selection flags
