@@ -1,7 +1,7 @@
 // qtractorMidiEditorForm.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -48,6 +48,8 @@
 
 #include "qtractorFileList.h"
 
+#include "qtractorClipCommand.h"
+
 #include <QFileDialog>
 #include <QMessageBox>
 #include <QActionGroup>
@@ -224,6 +226,14 @@ qtractorMidiEditorForm::qtractorMidiEditorForm (
 	m_pStatusModLabel->setAutoFillBackground(true);
 	pStatusBar->addPermanentWidget(m_pStatusModLabel);
 
+	// Session recording status.
+	m_pStatusRecLabel = new QLabel(tr("REC"));
+	m_pStatusRecLabel->setAlignment(Qt::AlignHCenter);
+	m_pStatusRecLabel->setMinimumSize(m_pStatusRecLabel->sizeHint() + pad);
+	m_pStatusRecLabel->setToolTip(tr("MIDI clip record state"));
+	m_pStatusRecLabel->setAutoFillBackground(true);
+	pStatusBar->addPermanentWidget(m_pStatusRecLabel);
+
 	// Sequence duration status.
 	m_pDurationLabel = new QLabel(tr("00:00:00.000"));
 	m_pDurationLabel->setAlignment(Qt::AlignHCenter);
@@ -232,6 +242,9 @@ qtractorMidiEditorForm::qtractorMidiEditorForm (
 	m_pDurationLabel->setAutoFillBackground(true);
 	pStatusBar->addPermanentWidget(m_pDurationLabel);
 
+	m_pRedPalette = new QPalette(pStatusBar->palette());
+	m_pRedPalette->setColor(QPalette::Window, Qt::red);
+
 	// Some actions surely need those
 	// shortcuts firmly attached...
 	addAction(m_ui.viewMenubarAction);
@@ -261,6 +274,9 @@ qtractorMidiEditorForm::qtractorMidiEditorForm (
 	QObject::connect(m_ui.fileUnlinkAction,
 		SIGNAL(triggered(bool)),
 		SLOT(fileUnlink()));
+	QObject::connect(m_ui.fileRecordExAction,
+		SIGNAL(triggered(bool)),
+		SLOT(fileRecordEx(bool)));
 	QObject::connect(m_ui.fileTrackInputsAction,
 		SIGNAL(triggered(bool)),
 		SLOT(fileTrackInputs()));
@@ -593,8 +609,19 @@ qtractorMidiEditorForm::qtractorMidiEditorForm (
 			pMainForm, SLOT(transportPanic()));
 	}
 
-	m_pEventTypeComboBox->setCurrentIndex(0);
-//	eventTypeChanged(0);
+	// Finally set initial editor type-params...
+	if (pOptions) {
+		m_pViewTypeComboBox->setCurrentIndex(pOptions->iMidiViewType);
+		const qtractorMidiControl::ControlType ctype
+			= m_pEventTypeGroup->controlTypeFromIndex(pOptions->iMidiEventType);
+		m_pEventTypeGroup->setControlType(ctype);
+		eventTypeChanged(ctype);
+		m_pEventTypeGroup->setControlParam(pOptions->iMidiEventParam);
+		viewTypeChanged(pOptions->iMidiViewType);
+	} else {
+		m_pEventTypeComboBox->setCurrentIndex(0);
+	//	eventTypeChanged(0);
+	}
 }
 
 
@@ -617,6 +644,10 @@ qtractorMidiEditorForm::~qtractorMidiEditorForm (void)
 
 	if (m_pEventTypeGroup)
 		delete m_pEventTypeGroup;
+
+	// Ditch rec-mode/red palette...
+	if (m_pRedPalette)
+		delete m_pRedPalette;
 }
 
 
@@ -699,6 +730,10 @@ void qtractorMidiEditorForm::closeEvent ( QCloseEvent *pCloseEvent )
 		pOptions->bMidiValueColor = m_ui.viewValueColorAction->isChecked();
 		pOptions->bMidiPreview = m_ui.viewPreviewAction->isChecked();
 		pOptions->bMidiFollow  = m_ui.viewFollowAction->isChecked();
+		// Save editor type-params...
+		pOptions->iMidiViewType = m_pViewTypeComboBox->currentIndex();
+		pOptions->iMidiEventType = m_pEventTypeComboBox->currentIndex();
+		pOptions->iMidiEventParam = m_pEventTypeGroup->controlParam();
 		// Save snap-per-beat setting...
 		pOptions->iMidiSnapPerBeat = m_pSnapPerBeatComboBox->currentIndex();
 		// Save snap-to-scale settings...
@@ -710,8 +745,13 @@ void qtractorMidiEditorForm::closeEvent ( QCloseEvent *pCloseEvent )
 		// Save the dock windows state.
 		pOptions->settings().setValue(
 			"/MidiEditor/Layout/DockWindows", saveState());
-		// And the main windows state?
+		// And this main windows state?
 		// pOptions->saveWidgetGeometry(this, true);
+		qtractorMidiClip *pMidiClip = midiClip();
+		if (pMidiClip) {
+			pMidiClip->setEditorPos(pos());
+			pMidiClip->setEditorSize(size());
+		}
 	}
 
 	// Remove this one from main-form list...
@@ -1041,6 +1081,20 @@ void qtractorMidiEditorForm::fileUnlink (void)
 }
 
 
+// Enter in clip record/ overdub mode.
+void qtractorMidiEditorForm::fileRecordEx ( bool bOn )
+{
+	qtractorMidiClip *pMidiClip = m_pMidiEditor->midiClip();
+	if (pMidiClip == NULL)
+		return;
+
+	// Start record/overdub the current clip, if any...
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession)
+		pSession->execute(new qtractorClipRecordExCommand(pMidiClip, bOn));
+}
+
+
 // File properties dialog.
 void qtractorMidiEditorForm::fileProperties (void)
 {
@@ -1631,14 +1685,20 @@ void qtractorMidiEditorForm::stabilizeForm (void)
 	m_ui.fileSaveAction->setEnabled(m_iDirtyCount > 0);
 	m_ui.fileUnlinkAction->setEnabled(pMidiClip && pMidiClip->isHashLinked());
 
+	m_ui.fileRecordExAction->setEnabled(pMidiClip != NULL);
+	m_ui.fileRecordExAction->setChecked(pTrack && pTrack->isClipRecordEx()
+		&& static_cast<qtractorMidiClip *> (pTrack->clipRecord()) == pMidiClip);
+
 	m_ui.fileTrackInputsAction->setEnabled(pTrack && pTrack->inputBus() != NULL);
 	m_ui.fileTrackOutputsAction->setEnabled(pTrack && pTrack->outputBus() != NULL);
 	m_ui.fileTrackPropertiesAction->setEnabled(pTrack != NULL);
 	m_ui.fileRangeSetAction->setEnabled(pTrack != NULL);
 	m_ui.fileLoopSetAction->setEnabled(pTrack != NULL);
 
-	m_pMidiEditor->updateUndoAction(m_ui.editUndoAction);
-	m_pMidiEditor->updateRedoAction(m_ui.editRedoAction);
+	// Update edit menu state...
+	qtractorCommandList *pCommands = m_pMidiEditor->commands();
+	pCommands->updateAction(m_ui.editUndoAction, pCommands->lastCommand());
+	pCommands->updateAction(m_ui.editRedoAction, pCommands->nextCommand());
 
 	const bool bSelected = m_pMidiEditor->isSelected();
 	const bool bSelectable = m_pMidiEditor->isSelectable();
@@ -1675,6 +1735,7 @@ void qtractorMidiEditorForm::stabilizeForm (void)
 		m_pTrackChannelLabel->clear();
 		m_pTrackNameLabel->clear();
 		m_pStatusModLabel->clear();
+		m_pStatusRecLabel->clear();
 		m_pDurationLabel->clear();
 		return;
 	}
@@ -1701,11 +1762,20 @@ void qtractorMidiEditorForm::stabilizeForm (void)
 	m_pFileNameLabel->setText(filename());
 	m_pTrackChannelLabel->setText(sTrackChannel.arg(trackChannel() + k));
 	m_pTrackNameLabel->setText(pSeq->name());
+
 	if (m_iDirtyCount > 0)
 		m_pStatusModLabel->setText(tr("MOD"));
 	else
 		m_pStatusModLabel->clear();
 
+	if (pTrack && pTrack->clipRecord() == pMidiClip) {
+		m_pStatusRecLabel->setText(tr("REC"));
+		m_pStatusRecLabel->setPalette(*m_pRedPalette);
+	} else {
+		m_pStatusRecLabel->clear();
+		m_pStatusRecLabel->setPalette(statusBar()->palette());
+	}
+
 	qtractorTimeScale *pTimeScale = m_pMidiEditor->timeScale();
 	m_pDurationLabel->setText(pTimeScale->textFromTick(
 		pTimeScale->tickFromFrame(m_pMidiEditor->offset()), true, pSeq->duration()));
@@ -1733,8 +1803,7 @@ void qtractorMidiEditorForm::stabilizeForm (void)
 		m_ui.transportLoopSetAction->setEnabled(
 			!bRolling && bSelectable);
 		m_ui.transportStopAction->setEnabled(bPlaying);
-		m_ui.transportRecordAction->setEnabled(
-			(!bLooping || !bPunching) && pSession->recordTracks() > 0);
+		m_ui.transportRecordAction->setEnabled(pSession->recordTracks() > 0);
 		m_ui.transportPunchAction->setEnabled(bPunching || bSelectable);
 		m_ui.transportPunchSetAction->setEnabled(bSelectable);
 		m_ui.transportRewindAction->setChecked(iRolling < 0);
