@@ -1,7 +1,7 @@
 // qtractorMidiEditorForm.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -39,6 +39,7 @@ class QContextMenuEvent;
 class QActionGroup;
 class QComboBox;
 class QLabel;
+class QPalette;
 
 
 //----------------------------------------------------------------------------
@@ -102,6 +103,7 @@ protected slots:
 	void fileSave();
 	void fileSaveAs();
 	void fileUnlink();
+	void fileRecordEx(bool bOn);
 	void fileTrackInputs();
 	void fileTrackOutputs();
 	void fileTrackProperties();
@@ -236,7 +238,10 @@ private:
 	QLabel *m_pFileNameLabel;
 	QLabel *m_pTrackChannelLabel;
 	QLabel *m_pStatusModLabel;
+	QLabel *m_pStatusRecLabel;
 	QLabel *m_pDurationLabel;
+
+	QPalette *m_pRedPalette;
 };
 
 
