@@ -1,7 +1,7 @@
 // qtractorOptions.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -87,6 +87,7 @@ void qtractorOptions::loadOptions (void)
 	bMessagesLimit  = m_settings.value("/MessagesLimit", true).toBool();
 	iMessagesLimitLines = m_settings.value("/MessagesLimitLines", 1000).toInt();
 	bConfirmRemove  = m_settings.value("/ConfirmRemove", true).toBool();
+	bConfirmArchive = m_settings.value("/ConfirmArchive", true).toBool();
 	bStdoutCapture  = m_settings.value("/StdoutCapture", true).toBool();
 	bCompletePath   = m_settings.value("/CompletePath", true).toBool();
 	bPeakAutoRemove = m_settings.value("/PeakAutoRemove", true).toBool();
@@ -124,6 +125,7 @@ void qtractorOptions::loadOptions (void)
 	bAutoBackward    = m_settings.value("/AutoBackward", false).toBool();
 	bContinuePastEnd = m_settings.value("/ContinuePastEnd", true).toBool();
 	iTransportMode   = m_settings.value("/TransportMode", 3).toInt();
+	bTimebase        = m_settings.value("/Timebase", true).toBool();
 	m_settings.endGroup();
 
 	// Audio rendering options group.
@@ -308,6 +310,9 @@ void qtractorOptions::loadOptions (void)
 	bMidiSnapZebra   = m_settings.value("/SnapZebra", false).toBool();
 	bMidiSnapGrid    = m_settings.value("/SnapGrid", false).toBool();
 	bMidiToolTips    = m_settings.value("/ToolTips", true).toBool();
+	iMidiViewType    = m_settings.value("/ViewType", 0).toInt();
+	iMidiEventType   = m_settings.value("/EventType", 0).toInt();
+	iMidiEventParam  = m_settings.value("/EventParam", 0).toInt();
 	iMidiSnapToScaleKey = m_settings.value("/SnapToScaleKey", 0).toInt();
 	iMidiSnapToScaleType = m_settings.value("/SnapToScaleType", 0).toInt();
 	m_settings.endGroup();
@@ -366,6 +371,7 @@ void qtractorOptions::saveOptions (void)
 	m_settings.setValue("/MessagesLimit", bMessagesLimit);
 	m_settings.setValue("/MessagesLimitLines", iMessagesLimitLines);
 	m_settings.setValue("/ConfirmRemove", bConfirmRemove);
+	m_settings.setValue("/ConfirmArchive", bConfirmArchive);
 	m_settings.setValue("/StdoutCapture", bStdoutCapture);
 	m_settings.setValue("/CompletePath", bCompletePath);
 	m_settings.setValue("/PeakAutoRemove", bPeakAutoRemove);
@@ -403,6 +409,7 @@ void qtractorOptions::saveOptions (void)
 	m_settings.setValue("/AutoBackward", bAutoBackward);
 	m_settings.setValue("/ContinuePastEnd", bContinuePastEnd);
 	m_settings.setValue("/TransportMode", iTransportMode);
+	m_settings.setValue("/Timebase", bTimebase);
 	m_settings.endGroup();
 
 	// Audio rendering options group.
@@ -578,6 +585,9 @@ void qtractorOptions::saveOptions (void)
 	m_settings.setValue("/SnapZebra", bMidiSnapZebra);
 	m_settings.setValue("/SnapGrid", bMidiSnapGrid);
 	m_settings.setValue("/ToolTips", bMidiToolTips);
+	m_settings.setValue("/ViewType", iMidiViewType);
+	m_settings.setValue("/EventType", iMidiEventType);
+	m_settings.setValue("/EventParam", iMidiEventParam);
 	m_settings.setValue("/SnapToScaleKey", iMidiSnapToScaleKey);
 	m_settings.setValue("/SnapToScaleType", iMidiSnapToScaleType);
 	m_settings.endGroup();
@@ -776,6 +786,8 @@ void qtractorOptions::saveWidgetGeometry ( QWidget *pWidget, bool bVisible )
 
 void qtractorOptions::loadComboBoxHistory ( QComboBox *pComboBox, int iLimit )
 {
+	const bool bBlockSignals = pComboBox->blockSignals(true);
+
 	// Load combobox list from configuration settings file...
 	m_settings.beginGroup("/History/" + pComboBox->objectName());
 
@@ -794,38 +806,43 @@ void qtractorOptions::loadComboBoxHistory ( QComboBox *pComboBox, int iLimit )
 	}
 
 	m_settings.endGroup();
+
+	pComboBox->blockSignals(bBlockSignals);
 }
 
 
 void qtractorOptions::saveComboBoxHistory ( QComboBox *pComboBox, int iLimit )
 {
-	int iCount = pComboBox->count();
+	const bool bBlockSignals = pComboBox->blockSignals(true);
 
 	// Add current text as latest item...
-	const QString& sCurrentText = pComboBox->currentText();
-	if (!sCurrentText.isEmpty()) {
-		int i = pComboBox->findText(sCurrentText);
-		if (i >= 0) {
+	const QString sCurrentText = pComboBox->currentText();
+	int iCount = pComboBox->count();
+	for (int i = 0; i < iCount; i++) {
+		const QString& sText = pComboBox->itemText(i);
+		if (sText == sCurrentText) {
 			pComboBox->removeItem(i);
 			--iCount;
+			break;
 		}
-		pComboBox->insertItem(0, sCurrentText);
-		++iCount;
 	}
-
-	// Take care of item limit...
-	while (iCount > iLimit)
+	while (iCount >= iLimit)
 		pComboBox->removeItem(--iCount);
+	pComboBox->insertItem(0, sCurrentText);
+	pComboBox->setCurrentIndex(0);
+	++iCount;
 
 	// Save combobox list to configuration settings file...
 	m_settings.beginGroup("/History/" + pComboBox->objectName());
 	for (int i = 0; i < iCount; ++i) {
 		const QString& sText = pComboBox->itemText(i);
 		if (sText.isEmpty())
-			continue;
+			break;
 		m_settings.setValue("/Item" + QString::number(i + 1), sText);
 	}
 	m_settings.endGroup();
+
+	pComboBox->blockSignals(bBlockSignals);
 }
 
 
