@@ -1,7 +1,7 @@
 // qtractorOptions.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -67,6 +67,7 @@ public:
 	bool    bMessagesLimit;
 	int     iMessagesLimitLines;
 	bool    bConfirmRemove;
+	bool    bConfirmArchive;
 	bool    bStdoutCapture;
 	bool    bCompletePath;
 	bool    bPeakAutoRemove;
@@ -97,6 +98,7 @@ public:
 	bool    bAutoBackward;
 	bool    bContinuePastEnd;
 	int     iTransportMode;
+	bool    bTimebase;
 
 	// Audio options...
 	QString sAudioCaptureExt;
@@ -255,6 +257,9 @@ public:
 	bool bMidiSnapZebra;
 	bool bMidiSnapGrid;
 	bool bMidiToolTips;
+	int  iMidiViewType;
+	int  iMidiEventType;
+	int  iMidiEventParam;
 	int  iMidiSnapToScaleKey;
 	int  iMidiSnapToScaleType;
 
