@@ -1,7 +1,7 @@
 // qtractorPlugin.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -219,16 +219,8 @@ bool qtractorPluginPath::open (void)
 #endif
 
 	QStringListIterator path_iter(path_list);
-	while (path_iter.hasNext()) {
-		const QDir dir(path_iter.next());
-		const QStringList& file_list = dir.entryList(QDir::Files);
-		QStringListIterator file_iter(file_list);
-		while (file_iter.hasNext()) {
-			const QString& sPath = dir.absoluteFilePath(file_iter.next());
-			if (QLibrary::isLibrary(sPath))
-				m_files.append(new qtractorPluginFile(sPath));
-		}
-	}
+	while (path_iter.hasNext())
+		addFiles(path_iter.next());
 
 	return (m_files.count() > 0);
 }
@@ -249,6 +241,27 @@ void qtractorPluginPath::setPaths (
 }
 
 
+// Recursive plugin file/path inventory method.
+void qtractorPluginPath::addFiles ( const QString& sPath )
+{
+	const QDir dir(sPath);
+	QDir::Filters filters = QDir::Files;
+	if (m_typeHint == qtractorPluginType::Vst)
+		filters = filters | QDir::AllDirs | QDir::NoDotAndDotDot;
+	const QFileInfoList& info_list = dir.entryInfoList(filters);
+	QListIterator<QFileInfo> info_iter(info_list);
+	while (info_iter.hasNext()) {
+		const QFileInfo& info = info_iter.next();
+		const QString& sFilename = info.absoluteFilePath();
+		if (info.isDir() && info.isReadable())
+			addFiles(sFilename);
+		else
+		if (QLibrary::isLibrary(sFilename))
+			m_files.append(new qtractorPluginFile(sFilename));
+	}
+}
+
+
 //----------------------------------------------------------------------------
 // qtractorPluginFile -- Plugin file library instance.
 //
@@ -259,7 +272,7 @@ bool qtractorPluginFile::open (void)
 	close();
 
 	// ATTN: Not really needed, as it would be
-	// loaded automagically on resolve()...
+	// loaded automagically on resolve(), but ntl...
 	if (!QLibrary::load())
 		return false;
 
@@ -281,14 +294,18 @@ void qtractorPluginFile::close (void)
 		return;
 
 	// Do the closing dance...
-#if 0 // defined(__WIN32__) || defined(_WIN32) || defined(WIN32)
+#if defined(__WIN32__) || defined(_WIN32) || defined(WIN32)
 	qtractorPluginFile_Function pfnFini
 		= (qtractorPluginFile_Function) QLibrary::resolve("_fini");
 	if (pfnFini)
 		(*pfnFini)();
 #endif
 
-	QLibrary::unload();
+	// ATTN: Might be really needed, as it would
+	// otherwise pile up hosing all available RAM
+	// until freed and unloaded on exit();
+	// nb. some VST might choke on auto-unload.
+	if (m_bAutoUnload) QLibrary::unload();
 }
 
 
@@ -304,8 +321,10 @@ bool qtractorPluginFile::getTypes ( qtractorPluginPath& path,
 	// Try DSSI plugin types first...
 	if (typeHint == qtractorPluginType::Any ||
 		typeHint == qtractorPluginType::Dssi) {
-		while ((pType
-			= qtractorDssiPluginType::createType(this, iIndex)) != NULL) {
+		while (true) {
+			pType = qtractorDssiPluginType::createType(this, iIndex);
+			if (pType == NULL)
+				break;
 			if (pType->open()) {
 				path.addType(pType);
 				pType->close();
@@ -325,8 +344,10 @@ bool qtractorPluginFile::getTypes ( qtractorPluginPath& path,
 	// Try LADSPA plugin types...
 	if (typeHint == qtractorPluginType::Any ||
 		typeHint == qtractorPluginType::Ladspa) {
-		while ((pType
-			= qtractorLadspaPluginType::createType(this, iIndex)) != NULL) {
+		while (true) {
+			pType = qtractorLadspaPluginType::createType(this, iIndex);
+			if (pType == NULL)
+				break;
 			if (pType->open()) {
 				path.addType(pType);
 				pType->close();
@@ -348,17 +369,20 @@ bool qtractorPluginFile::getTypes ( qtractorPluginPath& path,
 		typeHint == qtractorPluginType::Vst) {
 		// Need to look at the options...
 		qtractorOptions *pOptions = qtractorOptions::getInstance();
-		if (pOptions && pOptions->bDummyVstScan)
-			pType = qtractorDummyPluginType::createType(this);
-		else
-			pType = qtractorVstPluginType::createType(this);
-		if (pType) {
+		while (true) {
+			if (pOptions && pOptions->bDummyVstScan)
+				pType = qtractorDummyPluginType::createType(this, iIndex);
+			else
+				pType = qtractorVstPluginType::createType(this, iIndex);
+			if (pType == NULL)
+				break;
 			if (pType->open()) {
 				path.addType(pType);
 				pType->close();
 				++iIndex;
 			} else {
 				delete pType;
+				break;
 			}
 		}
 	}
@@ -454,26 +478,12 @@ qtractorPlugin *qtractorPluginFile::createPlugin (
 	}
 #endif
 
-#ifdef CONFIG_LV2
-	// Try LV2 plugin types...
-	if (typeHint == qtractorPluginType::Any ||
-		typeHint == qtractorPluginType::Lv2) {
-		qtractorLv2PluginType *pLv2Type
-			= qtractorLv2PluginType::createType(sFilename);
-		if (pLv2Type) {
-			if (pLv2Type->open())
-				return new qtractorLv2Plugin(pList, pLv2Type);
-			delete pLv2Type;
-		}
-	}
-#endif
-
 #ifdef CONFIG_VST
 	// Try VST plugin types...
 	if (typeHint == qtractorPluginType::Any ||
 		typeHint == qtractorPluginType::Vst) {
 		qtractorVstPluginType *pVstType
-			= qtractorVstPluginType::createType(pFile);
+			= qtractorVstPluginType::createType(pFile, iIndex);
 		if (pVstType) {
 			if (pVstType->open())
 				return new qtractorVstPlugin(pList, pVstType);
@@ -619,7 +629,7 @@ qtractorDummyPluginType *qtractorDummyPluginType::createType (
 	qtractorPluginFile *pFile, unsigned long iIndex, Hint typeHint )
 {
 	// Sanity check...
-	if (pFile == NULL)
+	if (pFile == NULL || iIndex > 0)
 		return NULL;
 
 	// Yep, most probably its a dummy plugin effect...
@@ -2054,6 +2064,8 @@ bool qtractorPluginList::loadElement (
 			qtractorPlugin::Values values;
 			qtractorMidiControl::Controllers controllers;
 			qtractorCurveFile cfile(qtractorPluginList::curveList());
+			QPoint posEditor;
+			QPoint posForm;
 			const QString& sTypeHint = ePlugin.attribute("type");
 			qtractorPluginType::Hint typeHint
 				= qtractorPluginType::hintFromText(sTypeHint);
@@ -2110,6 +2122,18 @@ bool qtractorPluginList::loadElement (
 					// Load plugin automation curves...
 					qtractorPlugin::loadCurveFile(&eParam, &cfile);
 				}
+				else
+				if (eParam.tagName() == "editor-pos") {
+					const QStringList& sxy = eParam.text().split(',');
+					posEditor.setX(sxy.at(0).toInt());
+					posEditor.setY(sxy.at(1).toInt());
+				}
+				else
+				if (eParam.tagName() == "form-pos") {
+					const QStringList& sxy = eParam.text().split(',');
+					posForm.setX(sxy.at(0).toInt());
+					posForm.setY(sxy.at(1).toInt());
+				}
 			}
 			qtractorPlugin *pPlugin
 				= qtractorPluginFile::createPlugin(this,
@@ -2141,6 +2165,8 @@ bool qtractorPluginList::loadElement (
 				pPlugin->applyCurveFile(&cfile);
 				pPlugin->setDirectAccessParamIndex(iDirectAccessParamIndex);
 				pPlugin->setActivated(bActivated); // Later's better!
+				pPlugin->setEditorPos(posEditor);
+				pPlugin->setFormPos(posForm);
 			} else {
 				qtractorMessageList::append(
 					QObject::tr("%1(%2): %3 plugin not found.")
@@ -2194,6 +2220,10 @@ bool qtractorPluginList::saveElement ( qtractorDocument *pDocument,
 	for (qtractorPlugin *pPlugin = qtractorPluginList::first();
 			pPlugin; pPlugin = pPlugin->next()) {
 
+		// Freeze form position, if currently visible...
+		if (pPlugin->isFormVisible())
+			pPlugin->setFormPos((pPlugin->form())->pos());
+
 		// Do freeze plugin state...
 		pPlugin->freezeConfigs();
 
@@ -2250,6 +2280,19 @@ bool qtractorPluginList::saveElement ( qtractorDocument *pDocument,
 			pPlugin->saveCurveFile(pDocument, &eCurveFile, &cfile);
 			ePlugin.appendChild(eCurveFile);
 		}
+		// Save editor position...
+		const QPoint& posEditor = pPlugin->editorPos();
+		if (!posEditor.isNull()) {
+			pDocument->saveTextElement("editor-pos",
+				QString::number(posEditor.x()) + ',' +
+				QString::number(posEditor.y()), &ePlugin);
+		}
+		const QPoint& posForm = pPlugin->formPos();
+		if (!posForm.isNull()) {
+			pDocument->saveTextElement("form-pos",
+				QString::number(posForm.x()) + ',' +
+				QString::number(posForm.y()), &ePlugin);
+		}
 
 		// Add this plugin...
 		pElement->appendChild(ePlugin);
