@@ -1,7 +1,7 @@
 // qtractorPlugin.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -186,8 +186,8 @@ class qtractorPluginFile : public QLibrary
 public:
 
 	// Constructor.
-	qtractorPluginFile(const QString& sFilename)
-		: QLibrary(sFilename) {}
+	qtractorPluginFile(const QString& sFilename, bool bAutoUnload = true)
+		: QLibrary(sFilename), m_bAutoUnload(bAutoUnload) {}
 
 	// Destructor.
 	~qtractorPluginFile()
@@ -200,6 +200,12 @@ public:
 	bool open();
 	void close();
 
+	// Auto-unload flag accessors.
+	void setAutoUnload(bool bAutoUnload)
+		{ m_bAutoUnload = bAutoUnload; }
+	bool isAutoUnload() const
+		{ return m_bAutoUnload; }
+
 	// Plugin type listing.
 	bool getTypes(qtractorPluginPath& path,
 		qtractorPluginType::Hint typeHint = qtractorPluginType::Any);
@@ -208,6 +214,11 @@ public:
 	static qtractorPlugin *createPlugin(qtractorPluginList *pList,
 		const QString& sFilename, unsigned long iIndex = 0,
 		qtractorPluginType::Hint typeHint = qtractorPluginType::Any);
+
+private:
+
+	// Instance variables.
+	bool m_bAutoUnload;
 };
 
 
@@ -254,6 +265,11 @@ public:
 	// Type list reset method.
 	void clear() { qDeleteAll(m_types); m_types.clear(); }
 
+protected:
+
+	// Recursive plugin file/path inventory method.
+	void addFiles(const QString& sPath);
+
 private:
 
 	// Instance variables.
@@ -530,6 +546,11 @@ public:
 
 	void updateEditorTitle();
 
+	void setEditorPos(const QPoint& pos)
+		{ m_posEditor = pos; }
+	const QPoint& editorPos() const
+		{ return m_posEditor; }
+
 	// An accessible list of observers.
 	const QList<qtractorPluginListItem *>& items() const
 		{ return m_items; }
@@ -543,6 +564,11 @@ public:
 	bool isFormVisible() const;
 	qtractorPluginForm *form();
 
+	void setFormPos(const QPoint& pos)
+		{ m_posForm = pos; }
+	const QPoint& formPos() const
+		{ return m_posForm; }
+
 	// Provisional preset accessors.
 	virtual QStringList presetList() const;
 
@@ -731,9 +757,11 @@ private:
 	// The plugin form reference.
 	qtractorPluginForm *m_pForm;
 	QString m_sPreset;
+	QPoint  m_posForm;
 
 	// GUI editor stuff.
 	QString m_sEditorTitle;
+	QPoint  m_posEditor;
 
 	// Plugin configuration (CLOB) stuff.
 	Configs m_configs;
