@@ -1,7 +1,7 @@
 // qtractorSession.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -899,8 +899,8 @@ void qtractorSession::insertTrack ( qtractorTrack *pTrack,
 }
 
 
-void qtractorSession::moveTrack ( qtractorTrack *pTrack,
-	qtractorTrack *pNextTrack )
+void qtractorSession::moveTrack (
+	qtractorTrack *pTrack, qtractorTrack *pNextTrack )
 {
 //	lock();
 
@@ -1153,7 +1153,8 @@ void qtractorSession::setPlaying ( bool bPlaying )
 		for (qtractorTrack *pTrack = m_tracks.first();
 				pTrack; pTrack = pTrack->next()) {
 			qtractorClip *pClipRecord = pTrack->clipRecord();
-			if (pClipRecord) {
+			if (pClipRecord && !pTrack->isClipRecordEx()) {
+				pTrack->setClipRecordStart(iClipStart);
 				pClipRecord->setClipStart(iClipStart);
 				// MIDI adjust to playing queue start...
 				if (pTrack->trackType() == qtractorTrack::Midi
@@ -1272,8 +1273,11 @@ void qtractorSession::setPlayHead ( unsigned long iFrame )
 	lock();
 	setPlaying(false);
 
-	if (m_pAudioEngine->jackClient())
-		jack_transport_locate(m_pAudioEngine->jackClient(), iFrame);
+	if (m_pAudioEngine->transportMode() & qtractorBus::Output) {
+		jack_client_t *pJackClient = m_pAudioEngine->jackClient();
+		if (pJackClient)
+			jack_transport_locate(pJackClient, iFrame);
+	}
 
 	seek(iFrame, true);
 
@@ -1292,8 +1296,8 @@ unsigned long qtractorSession::playHead (void) const
 
 
 // Session loop points accessors.
-void qtractorSession::setLoop ( unsigned long iLoopStart,
-	unsigned long iLoopEnd )
+void qtractorSession::setLoop (
+	unsigned long iLoopStart, unsigned long iLoopEnd )
 {
 	const bool bPlaying = isPlaying();
 	if (bPlaying && isRecording())
@@ -1496,6 +1500,8 @@ void qtractorSession::trackRecord (
 		iClipStart, iFrameTime);
 #endif
 
+	const qtractorTrack::TrackType trackType = pTrack->trackType();
+
 	// Just ditch the in-record clip...
 	if (!bRecord) {
 		pTrack->setClipRecord(NULL);
@@ -1503,7 +1509,7 @@ void qtractorSession::trackRecord (
 		if (recordTracks() < 1)
 			setRecording(false);
 		// One-down current tracks in record mode.
-		switch (pTrack->trackType()) {
+		switch (trackType) {
 		case qtractorTrack::Audio:
 			--m_iAudioRecord;
 			break;
@@ -1522,7 +1528,26 @@ void qtractorSession::trackRecord (
 		return;
 	}
 
-	switch (pTrack->trackType()) {
+	// Are we set record exclusive?
+	if (pTrack->isClipRecordEx()) {
+		// One-up current tracks in record mode.
+		switch (trackType) {
+		case qtractorTrack::Audio:
+			++m_iAudioRecord;
+			break;
+		case qtractorTrack::Midi:
+			++m_iMidiRecord;
+			break;
+		default:
+			break;
+		}
+		// Bail out.
+		return;
+	}
+
+	const bool bPlaying = isPlaying();
+
+	switch (trackType) {
 	case qtractorTrack::Audio:
 	{
 		qtractorAudioClip *pAudioClip = new qtractorAudioClip(pTrack);
@@ -1536,7 +1561,7 @@ void qtractorSession::trackRecord (
 			createFilePath(pTrack->trackName(),
 				qtractorAudioFileFactory::defaultExt()),
 			qtractorAudioFile::Write);
-		pTrack->setClipRecord(pAudioClip, iFrameTime);
+		pTrack->setClipRecord(pAudioClip);
 		// One-up audio tracks in record mode.
 		++m_iAudioRecord;
 		break;
@@ -1547,12 +1572,12 @@ void qtractorSession::trackRecord (
 		pMidiClip->setClipStart(iClipStart);
 		pMidiClip->openMidiFile(
 			createFilePath(pTrack->trackName(), "mid"),
-			qtractorMidiClip::defaultFormat(),
+			pTrack->midiChannel(),
 			qtractorMidiFile::Write);
-		pTrack->setClipRecord(pMidiClip, iFrameTime);
+		pTrack->setClipRecord(pMidiClip);
 		// MIDI adjust to playing queue start
 		// iif armed while already playing ...
-		if (isPlaying()) {
+		if (bPlaying) {
 			const unsigned long iTime = pMidiClip->clipStartTime();
 			const unsigned long iTimeStart = m_pMidiEngine->timeStart();
 			if (iTime > iTimeStart)
@@ -1566,9 +1591,13 @@ void qtractorSession::trackRecord (
 		break;
 	}
 
+	// Make sure the recording clip
+	// starts on some exact location...
+	pTrack->setClipRecordStart(bPlaying ? iFrameTime : iClipStart);
+
 #if 0
 	// Mute track as appropriate...
-	if (isPlaying())
+	if (bPlaying)
 		trackMute(pTrack, true);
 #endif
 }
@@ -1701,7 +1730,8 @@ bool qtractorSession::isAutoTimeStretch (void) const
 
 
 // Session special process cycle executive.
-void qtractorSession::process ( qtractorSessionCursor *pSessionCursor,
+void qtractorSession::process (
+	qtractorSessionCursor *pSessionCursor,
 	unsigned long iFrameStart, unsigned long iFrameEnd )
 {
 	const qtractorTrack::TrackType syncType = pSessionCursor->syncType();
