@@ -1,7 +1,7 @@
 // qtractorTrack.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -314,6 +314,8 @@ qtractorTrack::qtractorTrack ( qtractorSession *pSession, TrackType trackType )
 	m_pClipRecord = NULL;
 	m_iClipRecordStart = 0;
 
+	m_bClipRecordEx = false;
+
 	m_clips.setAutoDelete(true);
 
 	m_pSyncThread = NULL;
@@ -828,7 +830,6 @@ float qtractorTrack::prevGain (void) const
 	return (m_pMonitor ? m_pMonitor->prevGain() : 1.0f);
 }
 
-
 // Track stereo-panning accessor.
 void qtractorTrack::setPanning ( float fPanning )
 {
@@ -1086,18 +1087,20 @@ void qtractorTrack::removeClip ( qtractorClip *pClip )
 
 
 // Current clip on record (capture).
-void qtractorTrack::setClipRecord (
-	qtractorClip *pClipRecord, unsigned long iClipRecordStart )
+void qtractorTrack::setClipRecord ( qtractorClip *pClipRecord )
 {
-	if (m_pClipRecord)
+	if (!m_bClipRecordEx && m_pClipRecord)
 		delete m_pClipRecord;
 
 	m_pClipRecord = pClipRecord;
 
-	if (m_pSession && m_pSession->isPlaying())
-		m_iClipRecordStart = iClipRecordStart;
-	else
-		m_iClipRecordStart = (m_pClipRecord ? m_pClipRecord->clipStart() : 0);
+	if (m_pClipRecord == NULL) {
+		m_iClipRecordStart = 0;
+		if (m_bClipRecordEx) {
+			m_bClipRecordEx = false;
+			setRecord(false);
+		}
+	}
 }
 
 qtractorClip *qtractorTrack::clipRecord (void) const
@@ -1107,11 +1110,17 @@ qtractorClip *qtractorTrack::clipRecord (void) const
 
 
 // Current clip on record absolute start frame (capture).
+void qtractorTrack::setClipRecordStart ( unsigned long iClipRecordStart )
+{
+	m_iClipRecordStart = iClipRecordStart;
+}
+
 unsigned long qtractorTrack::clipRecordStart (void) const
 {
 	return m_iClipRecordStart;
 }
 
+
 unsigned long qtractorTrack::clipRecordEnd ( unsigned long iFrameTime ) const
 {
 	// HACK: Care of loop-recording + punch-in/out...
@@ -1143,10 +1152,23 @@ unsigned long qtractorTrack::clipRecordEnd ( unsigned long iFrameTime ) const
 		iClipRecordEnd -= m_iClipRecordStart;
 	if (m_pClipRecord)
 		iClipRecordEnd += m_pClipRecord->clipStart();
+
 	return iClipRecordEnd;
 }
 
 
+// Set current clip on exclusive recording.
+void qtractorTrack::setClipRecordEx ( bool bClipRecordEx )
+{
+	m_bClipRecordEx = bClipRecordEx;
+}
+
+bool qtractorTrack::isClipRecordEx (void) const
+{
+	return m_bClipRecordEx;
+}
+
+
 // Background color accessors.
 void qtractorTrack::setBackground ( const QColor& bg )
 {
@@ -1229,6 +1251,49 @@ void qtractorTrack::process ( qtractorClip *pClip,
 }
 
 
+// Freewheeling process cycle executive (needed for export).
+void qtractorTrack::process_export ( qtractorClip *pClip,
+	unsigned long iFrameStart, unsigned long iFrameEnd )
+{
+	// Track automation processing...
+	qtractorCurveList *pCurveList = curveList();
+	if (pCurveList && pCurveList->isProcess())
+		pCurveList->process(iFrameStart);
+
+	// Audio-buffers needs some preparation...
+	const unsigned int nframes = iFrameEnd - iFrameStart;
+	qtractorAudioMonitor *pAudioMonitor = NULL;
+	qtractorAudioBus *pOutputBus = NULL;
+	if (m_props.trackType == qtractorTrack::Audio) {
+		pAudioMonitor = static_cast<qtractorAudioMonitor *> (m_pMonitor);
+		pOutputBus = static_cast<qtractorAudioBus *> (m_pOutputBus);
+		if (pOutputBus)
+			pOutputBus->buffer_prepare(nframes);
+	}
+
+	// Playback...
+	if (!isMute() && (!m_pSession->soloTracks() || isSolo())) {
+		// Now, for every clip...
+		while (pClip && pClip->clipStart() < iFrameEnd) {
+			if (iFrameStart < pClip->clipStart() + pClip->clipLength())
+				pClip->process_export(iFrameStart, iFrameEnd);
+			pClip = pClip->next();
+		}
+	}
+
+	// Audio buffers needs monitoring and commitment...
+	if (pAudioMonitor && pOutputBus) {
+		// Plugin chain post-processing...
+		if (m_pPluginList->activated() > 0)
+			m_pPluginList->process(pOutputBus->buffer(), nframes);
+		// Monitor passthru...
+		pAudioMonitor->process(pOutputBus->buffer(), nframes);
+		// Actually render it...
+		pOutputBus->buffer_commit(nframes);
+	}
+}
+
+
 // Track special process record executive (audio recording only).
 void qtractorTrack::process_record (
 	unsigned long iFrameStart, unsigned long iFrameEnd )
@@ -1305,6 +1370,8 @@ void qtractorTrack::drawTrack ( QPainter *pPainter, const QRect& trackRect,
 	const QBrush brush(bg);
 #endif
 
+	qtractorClip *pClipRecordEx = (m_bClipRecordEx ? m_pClipRecord : NULL);
+
 	while (pClip) {
 		const unsigned long iClipStart = pClip->clipStart();
 		if (iClipStart > iTrackEnd)
@@ -1328,7 +1395,8 @@ void qtractorTrack::drawTrack ( QPainter *pPainter, const QRect& trackRect,
 			pPainter->setPen(pen);
 			pPainter->setBrush(brush);
 			// Draw the clip...
-			pClip->drawClip(pPainter, QRect(x, y, w - x, h), iClipOffset);
+			const QRect clipRect(x, y, w - x, h);
+			pClip->drawClip(pPainter, clipRect, iClipOffset);
 		#if 0
 			// Draw the clip selection...
 			if (pClip->isClipSelected()) {
@@ -1349,6 +1417,8 @@ void qtractorTrack::drawTrack ( QPainter *pPainter, const QRect& trackRect,
 				pPainter->fillRect(QRect(x, y, w - x, h), QColor(0, 0, 255, 120));
 			}
 		#endif
+			if (pClip == pClipRecordEx)
+				pPainter->fillRect(clipRect, QColor(255, 0, 0, 60));
 		}
 		pClip = pClip->next();
 	}
