@@ -1,7 +1,7 @@
 // qtractorVstPlugin.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -30,12 +30,15 @@
 #include "qtractorMidiBuffer.h"
 
 #include "qtractorSession.h"
+
+#ifdef QTRACTOR_VST_EDITOR_TOOL
 #include "qtractorOptions.h"
+#endif
 
 #include <QApplication>
 #include <QFileDialog>
 #include <QFileInfo>
-#include <QWidget>
+
 
 #if QT_VERSION < 0x040500
 namespace Qt {
@@ -186,8 +189,8 @@ public:
 		} *pRect;
 
 		if (m_pVstPlugin->vst_dispatch(0, effEditGetRect, 0, 0, &pRect, 0.0f)) {
-			int w = pRect->right - pRect->left;
-			int h = pRect->bottom - pRect->top;
+			const int w = pRect->right - pRect->left;
+			const int h = pRect->bottom - pRect->top;
 			if (w > 0 && h > 0)
 				QWidget::setFixedSize(w, h);
 		}
@@ -215,7 +218,6 @@ public:
 
 		if (m_pVstPlugin) {
 			m_pVstPlugin->vst_dispatch(0, effEditClose, 0, 0, NULL, 0.0f);
-		//	m_pVstPlugin->setEditorVisible(false);
 			m_pVstPlugin = NULL;
 		}
 
@@ -225,7 +227,6 @@ public:
 	}
 
 #if defined(Q_WS_X11)
-
 	// Local X11 event filter.
 	bool x11EventFilter(XEvent *pEvent)
 	{
@@ -252,7 +253,6 @@ public:
 			return false;
 		}
 	}
-
 #endif
 
 	qtractorVstPlugin *plugin() const
@@ -280,16 +280,16 @@ protected:
 			m_pVstPlugin->closeEditor();
 	}
 
+#if defined(Q_WS_X11)
 	void moveEvent(QMoveEvent *pMoveEvent)
 	{
 		QWidget::moveEvent(pMoveEvent);
-	#if defined(Q_WS_X11)
 		if (m_wVstEditor) {
 			XMoveWindow(m_pDisplay, m_wVstEditor, 0, 0);
 		//	QWidget::update();
 		}
-	#endif
 	}
+#endif
 
 private:
 
@@ -317,7 +317,7 @@ public:
 	Effect(AEffect *pVstEffect = NULL) : m_pVstEffect(pVstEffect) {}
 
 	// Specific methods.
-	bool open(qtractorPluginFile *pFile);
+	bool open(qtractorPluginFile *pFile, unsigned long iIndex);
 	void close();
 
 	// Specific accessors.
@@ -334,8 +334,13 @@ private:
 };
 
 
+// Current working VST Shell identifier.
+static int g_iVstShellCurrentId = 0;
+
+
 // Specific methods.
-bool qtractorVstPluginType::Effect::open ( qtractorPluginFile *pFile )
+bool qtractorVstPluginType::Effect::open (
+	qtractorPluginFile *pFile, unsigned long iIndex )
 {
 	// Do we have an effect descriptor already?
 	if (m_pVstEffect == NULL)
@@ -343,12 +348,50 @@ bool qtractorVstPluginType::Effect::open ( qtractorPluginFile *pFile )
 	if (m_pVstEffect == NULL)
 		return false;
 
+	// Check whether it's a VST Shell...
+	const int categ = vst_dispatch(effGetPlugCategory, 0, 0, NULL, 0.0f);
+	if (categ == kPlugCategShell) {
+		int id = 0;
+		char buf[40];
+		unsigned long i = 0;
+		for ( ; iIndex >= i; ++i) {
+			buf[0] = (char) 0;
+			id = vst_dispatch(effShellGetNextPlugin, 0, 0, (void *) buf, 0.0f);
+			if (id == 0 || !buf[0])
+				break;
+		}
+		// Check if we're actually the intended plugin...
+		if (i < iIndex || id == 0 || !buf[0]) {
+			m_pVstEffect = NULL;
+			return false;
+		}
+		// Make it known...
+		g_iVstShellCurrentId = id;
+		// Re-allocate the thing all over again...
+		m_pVstEffect = qtractorVstPluginType::vst_effect(pFile);
+		// Not needed anymore, hopefully...
+		g_iVstShellCurrentId = 0;
+		// Don't go further if failed...
+		if (m_pVstEffect == NULL)
+			return false;
+	#ifdef CONFIG_DEBUG//_0
+		qDebug("AEffect[%p]::vst_shell(%lu) id=0x%x name=\"%s\"",
+			m_pVstEffect, i, id, buf);
+	#endif
+	}
+	else
+	// Not a VST Shell plugin...
+	if (iIndex > 0) {
+		m_pVstEffect = NULL;
+		return false;
+	}
+
 #ifdef CONFIG_DEBUG_0
-	qDebug("AEffect[%p]::open(%p)", m_pVstEffect, pFile);
+	qDebug("AEffect[%p]::open(%p, %lu)", m_pVstEffect, pFile, iIndex);
 #endif
 
-//	vst_dispatch(effIdentify, 0, 0, NULL, 0);
 	vst_dispatch(effOpen, 0, 0, NULL, 0.0f);
+//	vst_dispatch(effIdentify, 0, 0, NULL, 0);
 //	vst_dispatch(effMainsChanged, 0, 0, NULL, 0.0f);
 
 	return true;
@@ -397,7 +440,7 @@ bool qtractorVstPluginType::open (void)
 	// Do we have an effect descriptor already?
 	if (m_pEffect == NULL)
 		m_pEffect = new Effect();
-	if (!m_pEffect->open(file()))
+	if (!m_pEffect->open(file(), index()))
 		return false;
 
 	AEffect *pVstEffect = m_pEffect->vst_effect();
@@ -454,6 +497,12 @@ bool qtractorVstPluginType::open (void)
 	m_bConfigure = (pVstEffect->flags & effFlagsProgramChunks);
 	m_bEditor    = (pVstEffect->flags & effFlagsHasEditor);
 
+	// HACK: Some native VST plugins with a GUI editor
+	// need to skip explicit shared library unloading,
+	// on close, in order to avoid mysterious crashes
+	// later on session and/or application exit.
+	if (m_bEditor) file()->setAutoUnload(false);
+
 	return true;
 }
 
@@ -476,7 +525,7 @@ void qtractorVstPluginType::close (void)
 
 // Factory method (static)
 qtractorVstPluginType *qtractorVstPluginType::createType (
-	qtractorPluginFile *pFile )
+	qtractorPluginFile *pFile, unsigned long iIndex )
 {
 	// Sanity check...
 	if (pFile == NULL)
@@ -488,7 +537,7 @@ qtractorVstPluginType *qtractorVstPluginType::createType (
 		return NULL;
 
 	// Yep, most probably its a valid plugin effect...
-	return new qtractorVstPluginType(pFile, new Effect(pVstEffect));
+	return new qtractorVstPluginType(pFile, iIndex, new Effect(pVstEffect));
 }
 
 
@@ -648,12 +697,15 @@ void qtractorVstPlugin::setChannels ( unsigned short iChannels )
 
 	// Close old instances, all the way...
 	if (m_ppEffects) {
+		qtractorVstPluginType::Effect *pEffect;
 		for (unsigned short i = 1; i < iOldInstances; ++i) {
-			g_vstPlugins.remove(m_ppEffects[i]->vst_effect());
-			m_ppEffects[i]->close();
-			delete m_ppEffects[i];
+			pEffect = m_ppEffects[i];
+			g_vstPlugins.remove(pEffect->vst_effect());
+			pEffect->close();
+			delete pEffect;
 		}
-		g_vstPlugins.remove(m_ppEffects[0]->vst_effect());
+		pEffect = m_ppEffects[0];
+		g_vstPlugins.remove(pEffect->vst_effect());
 		delete [] m_ppEffects;
 		m_ppEffects = NULL;
 	}
@@ -675,22 +727,15 @@ void qtractorVstPlugin::setChannels ( unsigned short iChannels )
 	g_vstPlugins.insert(m_ppEffects[0]->vst_effect(), this);
 	for (unsigned short i = 1; i < iInstances; ++i) {
 		m_ppEffects[i] = new qtractorVstPluginType::Effect();
-		m_ppEffects[i]->open(pVstType->file());
+		m_ppEffects[i]->open(pVstType->file(), pVstType->index());
 		g_vstPlugins.insert(m_ppEffects[i]->vst_effect(), this);
 	}
 
-	// (Re)issue all configuration as needed...
-	realizeConfigs();
-	realizeValues();
-
-	// But won't need it anymore.
-	releaseConfigs();
-	releaseValues();
-
 	// Setup all those instance alright...
 	for (unsigned short i = 0; i < iInstances; ++i) {
 		// And now all other things as well...
 		qtractorVstPluginType::Effect *pEffect = m_ppEffects[i];
+	//	pEffect->vst_dispatch(effOpen, 0, 0, NULL, 0.0f);
 		pEffect->vst_dispatch(effSetSampleRate, 0, 0, NULL, float(sampleRate()));
 		pEffect->vst_dispatch(effSetBlockSize,  0, bufferSize(), NULL, 0.0f);
 	//	pEffect->vst_dispatch(effSetProgram, 0, 0, NULL, 0.0f);
@@ -703,6 +748,14 @@ void qtractorVstPlugin::setChannels ( unsigned short iChannels )
 	#endif
 	}
 
+	// (Re)issue all configuration as needed...
+	realizeConfigs();
+	realizeValues();
+
+	// But won't need it anymore.
+	releaseConfigs();
+	releaseValues();
+
 	// (Re)activate instance if necessary...
 	setActivated(bActivated);
 }
@@ -916,6 +969,10 @@ void qtractorVstPlugin::freezeConfigs (void)
 	// saving plugin's state and before parameter values.
 	updateParamValues(false);
 
+	// Also, update current editor position...
+	if (m_pEditorWidget && m_pEditorWidget->isVisible())
+		setEditorPos(m_pEditorWidget->pos());
+
 	if (!type()->isConfigure())
 		return;
 
@@ -978,8 +1035,12 @@ void qtractorVstPlugin::openEditor ( QWidget */*pParent*/ )
 {
 	// Is it already there?
 	if (m_pEditorWidget) {
-		if (!m_pEditorWidget->isVisible())
+		if (!m_pEditorWidget->isVisible()) {
+			const QPoint& posEditor = editorPos();
+			if (!posEditor.isNull())
+				m_pEditorWidget->move(posEditor);
 			m_pEditorWidget->show();
+		}
 		m_pEditorWidget->raise();
 		m_pEditorWidget->activateWindow();
 		return;
@@ -989,6 +1050,9 @@ void qtractorVstPlugin::openEditor ( QWidget */*pParent*/ )
 	qDebug("qtractorVstPlugin[%p]::openEditor()", this);
 #endif
 
+	// Make sure it's not closed...
+	m_bEditorClosed = false;
+
 	// Create the new parent frame...
 	Qt::WindowFlags wflags = Qt::Window
 		| Qt::CustomizeWindowHint
@@ -996,10 +1060,11 @@ void qtractorVstPlugin::openEditor ( QWidget */*pParent*/ )
 		| Qt::WindowSystemMenuHint
 		| Qt::WindowMinMaxButtonsHint
 		| Qt::WindowCloseButtonHint;
+#ifdef QTRACTOR_VST_EDITOR_TOOL
 	qtractorOptions *pOptions = qtractorOptions::getInstance();
 	if (pOptions && pOptions->bKeepToolsOnTop)
 		wflags |= Qt::Tool;
-
+#endif
 	m_pEditorWidget = new EditorWidget(NULL, wflags);
 	m_pEditorWidget->open(this);
 }
@@ -1020,6 +1085,8 @@ void qtractorVstPlugin::closeEditor (void)
 	qDebug("qtractorVstPlugin[%p]::closeEditor()", this);
 #endif
 
+	setEditorVisible(false);
+
 	// Close the parent widget, if any.
 	delete m_pEditorWidget;
 	m_pEditorWidget = NULL;
@@ -1045,7 +1112,15 @@ void qtractorVstPlugin::idleEditor (void)
 // GUI editor visibility state.
 void qtractorVstPlugin::setEditorVisible ( bool bVisible )
 {
-	if (m_pEditorWidget) m_pEditorWidget->setVisible(bVisible);
+	if (m_pEditorWidget) {
+		if (bVisible) {
+			const QPoint& posEditor = editorPos();
+			if (!posEditor.isNull())
+				m_pEditorWidget->move(posEditor);
+		}
+		else setEditorPos(m_pEditorWidget->pos());
+		m_pEditorWidget->setVisible(bVisible);
+	}
 }
 
 bool qtractorVstPlugin::isEditorVisible (void) const
@@ -1456,6 +1531,7 @@ static VstIntPtr VSTCALLBACK qtractorVstPlugin_HostCallback ( AEffect *effect,
 
 	case audioMasterCurrentId:
 		VST_HC_DEBUG("audioMasterCurrentId");
+		ret = (VstIntPtr) g_iVstShellCurrentId;
 		break;
 
 	case audioMasterIdle:
